""" Automation of memory cache suite

:requirement: inmemory_cache
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved

"""
import time
import pytest
import subprocess
from sssd.testlib.common.utils import sssdTools
from constants import ds_instance_name


def find_logs(multihost, log_name, string_name):
    """This function will find strings in a log file
    log_name: Absolute path of log where the search will happen.
    string_name: String to search in the log file.
    """
    log_str = multihost.client[0].get_file_contents(log_name).decode('utf-8')
    assert string_name in log_str


@pytest.fixture(scope='function')
def common_sssd_setup(multihost):
    """
    This is common sssd setup used in this test suit.
    """
    tools = sssdTools(multihost.client[0])
    tools.sssd_conf("nss", {'debug_level': '9'}, action='update')
    ldap_params = {'use_fully_qualified_names': False}
    tools.sssd_conf(f'domain/{ds_instance_name}', ldap_params)
    tools.clear_sssd_cache()
    client = multihost.client[0]
    client.run_command("rm -vf /tmp/straceuser*")


def clear_only_domain_log(multihost):
    "This function will clear domain logs"
    client = multihost.client[0]
    log_sssd = f'/var/log/sssd/sssd_{ds_instance_name}.log'
    client.run_command(f'> {log_sssd}')


@pytest.mark.tier1_4
@pytest.mark.usefixtures('setup_sssd',
                         'create_posix_usersgroups')
@pytest.mark.sss_cache
class TestPasswordPolicy(object):
    """
    This is test case class for sssd memory cache suite
    """
    @staticmethod
    def test_bz789507(multihost, backupsssdconf, common_sssd_setup):
        """
        :title: SSSD should provide fast in memory cache bz789507
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=789507
        :id: 19dfa7ae-96e9-11ed-adf3-845cf3eff344
        :steps:
            1. For new user foo1 run command getent passwd foo1,
                cache should be stored for this operation.
            2. For new group ldapusers run command getent group ldapusers,
                cache should be stored for this operation.
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        client = multihost.client[0]
        client.run_command("strace -o /tmp/straceuser1 getent passwd foo1")
        log_name = "/tmp/straceuser1"
        log_ssd = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        find_logs(multihost, log_name, "/var/lib/sss/pipes/nss")
        find_logs(multihost, log_name, "/var/lib/sss/mc/passwd")
        find_logs(multihost, log_ssd, "Got request for ")
        find_logs(multihost, log_ssd, "uid=foo1")
        clear_only_domain_log(multihost)
        client.run_command("strace -o /tmp/stracegroup1 getent group ldapusers")
        log_name = "/tmp/stracegroup1"
        find_logs(multihost, log_name, "/var/lib/sss/pipes/nss")
        find_logs(multihost, log_name, "/var/lib/sss/mc/group")
        find_logs(multihost, log_ssd, "Got request for ")
        find_logs(multihost, log_ssd, "cn=ldapusers")

    @staticmethod
    def test_maxage(multihost, common_sssd_setup):
        """
        :title: Data is directly pulled from inmemory cache
            without opening the pipe to the NSS responder
        :id: 31217168-96e9-11ed-b69f-845cf3eff344
        :steps:
            1. Clear sssd domain logs
            2. Make sure /var/lib/sss/pipes/nss was not called while cache is there
            3. sssd should not send request to master server while cache is there,
                by checking that new request was not sent to master server.
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
        """
        client = multihost.client[0]
        file_ssd = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        client.run_command("strace -o /tmp/straceuser1 getent passwd foo1")
        client.run_command("strace -o /tmp/stracegroup1 getent group ldapusers")
        clear_only_domain_log(multihost)
        client.run_command("strace -o /tmp/straceuser2 getent passwd foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/straceuser2", "/var/lib/sss/pipes/nss")
        with pytest.raises(AssertionError):
            find_logs(multihost, file_ssd, "Got request for ")
        with pytest.raises(AssertionError):
            find_logs(multihost, file_ssd, "uid=foo1")
        find_logs(multihost, "/tmp/straceuser2", "/var/lib/sss/mc/passwd")
        clear_only_domain_log(multihost)
        client.run_command("strace -o /tmp/stracegroup2 getent group ldapusers")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/stracegroup2", "/var/lib/sss/pipes/nss")
        with pytest.raises(AssertionError):
            find_logs(multihost, file_ssd, "Got request for ")
        with pytest.raises(AssertionError):
            find_logs(multihost, file_ssd, "cn=ldapusers")
        find_logs(multihost, "/tmp/stracegroup2", "/var/lib/sss/mc/group")

    @staticmethod
    def test_invalidate_inmemory_cache(multihost, common_sssd_setup):
        """
        :title: sss cache invalidates the InMemory cache
        :id: 92ee87f8-96eb-11ed-a172-845cf3eff344
        :steps:
            1. Make sure sss cache invalidates the InMemory cache.
                Force invalidation of cached data.
        :expectedresults:
            1. Forcing up-to-date data into SSSD cache.
        """
        clear_only_domain_log(multihost)
        log_sssd = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        client = multihost.client[0]
        client.run_command(f"sss_cache -U --domain={ds_instance_name}")
        client.run_command("strace -o /tmp/straceuser3 getent passwd foo1")
        find_logs(multihost, "/tmp/straceuser3", "/var/lib/sss/pipes/nss")
        find_logs(multihost, "/tmp/straceuser3", "/var/lib/sss/mc/passwd")
        find_logs(multihost, log_sssd, "Got request for ")
        find_logs(multihost, log_sssd, "uid=foo1")
        clear_only_domain_log(multihost)
        client.run_command(f"sss_cache -G --domain={ds_instance_name}")
        client.run_command("strace -o /tmp/stracegroup3 getent group ldapusers")
        find_logs(multihost, "/tmp/stracegroup3", "/var/lib/sss/pipes/nss")
        find_logs(multihost, "/tmp/stracegroup3", "/var/lib/sss/mc/group")
        find_logs(multihost, log_sssd, "Got request for ")
        find_logs(multihost, log_sssd, "cn=ldapusers")

    @staticmethod
    def test_bz867933(multihost, common_sssd_setup):
        """
        :title: sss cache invalidates the InMemory cache when sssd is stopped bz867933
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=867933
        :id: 0db6d2e0-96ee-11ed-bbb9-845cf3eff344
        :steps:
            1. Make the API to invalidate the cache public
            2. In the sss_cache tool, carefully detect if the sss_nss process is running
            3. if the sss_nss process is running, proceed with sending the signal
            4. if the sss_nss process is not running, invalidate the memcache.
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
            4. Should succeed
        """
        client = multihost.client[0]
        log_sssd = f'/var/log/sssd/sssd_{ds_instance_name}.log'
        client.run_command("service sssd stop")
        client.run_command(f"sss_cache -U --domain={ds_instance_name}")
        client.run_command(f"sss_cache -G --domain={ds_instance_name}")
        client.run_command("service sssd start")
        clear_only_domain_log(multihost)
        client.run_command("getent passwd foo1")
        client.run_command("getent group ldapusers")
        find_logs(multihost, log_sssd, "Got request for ")
        find_logs(multihost, log_sssd, "uid=foo1")
        find_logs(multihost, log_sssd, "cn=ldapusers")

    @staticmethod
    def test_memcache_timeout(multihost, backupsssdconf, common_sssd_setup):
        """
        :title: Set value of option memcache timeout to 0 and expire InMemory cache immediately
        :id: 19dfa7ae-96e9-11ed-adf3-845cf3eff344
        :steps:
            1. Set value of option memcache timeout to 0 and expire InMemory cache immediately
            2. Validate the user using sss cache
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        client = multihost.client[0]
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf("nss", {'memcache_timeout': '0'}, action='update')
        tools.clear_sssd_cache()
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(2)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(5)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(5)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")

    @staticmethod
    def test_bz1484376(multihost, backupsssdconf, common_sssd_setup):
        """
        :title: Add a configuration option to SSSD to disable the memory cache bz1484376
                Set value of option Memcache timeout to 35
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1484376
        :id: b64237b4-96ef-11ed-adb1-845cf3eff344
        :steps:
            1. Add a configuration option to SSSD to disable the memory cache
            2. Set memcache_timeout=0
            3. Set value of option Memcache timeout to 35
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
        """
        client = multihost.client[0]
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf("nss", {'memcache_timeout': '0'}, action='update')
        tools.clear_sssd_cache()
        client.run_command("rm -rvf /var/lib/sss/mc/*")
        client.run_command("systemctl restart sssd")
        # MC directory is empty, result as expected.
        assert client.run_command("ls -A /var/lib/sss/mc/").stdout_text == ''
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf("nss", {'memcache_timeout': '35'}, action='update')
        tools.clear_sssd_cache()
        clear_only_domain_log(multihost)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        date_start = int(client.run_command('date "+%s"').stdout_text.split()[0])
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(10)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(10)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(10)
        elapsed_second = int(client.run_command('date "+%s"').stdout_text.split()[0]) - date_start
        if elapsed_second < 35:
            client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
            with pytest.raises(AssertionError):
                find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(6)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")

    @staticmethod
    def test_time_out_300(multihost, backupsssdconf, common_sssd_setup):
        """
        :title: Set default value of option Memcache timeout to 300
        :id: a0b9ee92-96fd-11ed-bc4e-845cf3eff344
        :steps:
            1. Set default value of option Memcache timeout to 300
            2. Validate the user using sss cache
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        client = multihost.client[0]
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf("nss", {'memcache_timeout': ''}, action='delete')
        tools.clear_sssd_cache()
        clear_only_domain_log(multihost)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(100)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(100)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")
        time.sleep(101)
        client.run_command("strace -o /tmp/stracefoo1 getent passwd foo1")
        find_logs(multihost, "/tmp/stracefoo1", "/var/lib/sss/pipes/nss")

    @staticmethod
    def test_default_behaviour(multihost, backupsssdconf, common_sssd_setup):
        """
        :title: Verify default behaviour of Cache Initgroups feature
        :id: 975d9060-96fd-11ed-b9f7-845cf3eff344
        :steps:
            1. Set Memcache timeout default
            2. Set Memcache timeout 0 and expire cache immediately
            3. Invalidate the user using sss cache
            4. Invalidate the group using sss cache
            5. Stop sssd service and verify the cache
        :expectedresults:
            1. Should succeed
            2. Should succeed
            3. Should succeed
            4. Should succeed
            5. Should succeed
        """
        client = multihost.client[0]
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf("nss", {'memcache_timeout': ''}, action='delete')
        tools.clear_sssd_cache()
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/mc/initgroups")
        time.sleep(10)
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        # Set Memcache timeout 0 and expire cache immediately
        tools.sssd_conf("nss", {'memcache_timeout': '0'}, action='update')
        tools.clear_sssd_cache()
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        time.sleep(3)
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        tools.clear_sssd_cache()
        # Invalidate the user using sss cache
        client.run_command("id foo1")
        client.run_command("sss_cache -u foo1")
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/mc/initgroups")
        # Invalidate the group using sss cache
        tools.clear_sssd_cache()
        client.run_command("id foo1")
        client.run_command(f"sss_cache -G --domain={ds_instance_name}")
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/mc/initgroups")
        # Stop sssd service and verify the cache
        tools.sssd_conf("nss", {'memcache_timeout': ''}, action='delete')
        tools.clear_sssd_cache()
        client.run_command("id foo1")
        client.run_command("service sssd stop")
        client.run_command("strace -o /tmp/tracefile id --groups foo1")
        with pytest.raises(AssertionError):
            find_logs(multihost, "/tmp/tracefile", "/var/lib/sss/pipes/nss")
        client.run_command("sss_cache -u foo1")
        with pytest.raises(subprocess.CalledProcessError):
            client.run_command("id foo1")
        client.run_command("service sssd start")



""" This provides common functions for ipa """
from sssd.testlib.common.exceptions import SSSDException
from sssd.testlib.common.expect import pexpect_ssh
from sssd.testlib.common.exceptions import SSHLoginException
import subprocess
import pexpect


class ipaTools(object):
    """ Collection of assorted functions for ipa to be used in fixtures
        Attributes:
          Host(obj: `Multihost object type`): Multihost Object
    """
    def __init__(self, Host):
        """ Initialize multihost """
        self.multihost = Host

    def install_common_pkgs(self):
        """ Install common required packages """
        pkgs = 'ldb-tools tcpdump wireshark-cli expect python3-libsss_nss_idmap'
        if '8.' in self.multihost.distro:
            enable_idm1 = "dnf -y module reset idm"
            self.multihost.run_command(enable_idm1)
            enable_idm2 = "dnf -y module enable idm:DL1"
            self.multihost.run_command(enable_idm2)
            enable_idm3 = "dnf -y module install idm:DL1/client"
            self.multihost.run_command(enable_idm3)
        if 'Fedora' in self.multihost.distro:
            client_pkgs = ' freeipa-client'
            pkgs = pkgs + client_pkgs
        self.multihost.package_mgmt(pkgs, action='install')

    def setup_chrony(self, ntp_server='pool.ntp.org'):
        """ Setup chrony
            Attributes:
            ntp_server(str): NTP server. Default ntp_server is pool.ntp.org
            Return: bool
        """
        stop_chrony = 'systemctl stop chronyd'
        self.multihost.run_command(stop_chrony)
        cmd = "chronyd -q 'server %s iburst'" % ntp_server
        try:
            self.multihost.run_command(cmd)
        except subprocess.CalledProcessError:
            raise SSSDException("Unable to set ntp server")

        start_chrony = 'systemctl start chronyd'
        try:
            self.multihost.run_command(start_chrony)
        except subprocess.CalledProcessError:
            return False
        else:
            return True

    def get_default_nw_uuid(self):
        """ Get default network interface uuid"""
        nmcli_cmd = "nmcli con show --active"
        cmd = self.multihost.run_command(nmcli_cmd, raiseonerr=False)
        conn_list = cmd.stdout_text.split('\n')[1].split(' ')
        filtered_list = list(filter(None, conn_list))
        return filtered_list[2]

    def get_interface_ip(self, uuid):
        """ Get IP Address associated with interface
            using nmcli
        """
        if uuid is None:
            return False
        nmcli_cmd = "nmcli -f IP4.ADDRESS conn show %s" % uuid
        cmd = self.multihost.run_command(nmcli_cmd, raiseonerr=False)
        if cmd.returncode == 0:
            ipaddr = cmd.stdout_text.split()[1].split('/')[0]
            return ipaddr
        else:
            return False

    def add_hbac_rule(self, rulename, username, hostname,
                      service, group=False):
        """ Add IPA hbac rule """
        # add rule
        add_rule = "ipa hbacrule-add %s" % rulename
        # add user
        if group:
            add_user = "ipa hbacrule-add-user --groups %s %s" % (username,
                                                                 rulename)
        else:
            add_user = "ipa hbacrule-add-user --users %s %s" % (username,
                                                                rulename)
        # add host
        add_host = "ipa hbacrule-add-host --hosts %s %s " % (hostname,
                                                             rulename)
        # add service
        add_service = "ipa hbacrule-add-service --hbacsvcs=%s %s" % (service,
                                                                     rulename)
        cmd_list = [add_rule, add_user, add_host, add_service]
        for cmd in cmd_list:
            ret = self.multihost.run_command(cmd, raiseonerr=False)
            if ret.returncode != 0:
                raise SSSDException(ret.stderr_text)

    def del_hbac_rule(self, rulename):
        """ Delete hbac rule """
        # delete rule
        del_rule = "ipa hbacrule-del %s" % rulename
        ret = self.multihost.run_command(del_rule, raiseonerr=False)
        if ret.returncode != 0:
            raise SSSDException(ret.stderr_text)

    def ssh_login(self, username, password, host, command=None):
        """ SSH login to host """
        pxssh = pexpect_ssh(host, username, password, debug=False)
        try:
            pxssh.login()
        except SSHLoginException:
            return False
        except pexpect.exceptions.EOF:
            return False
        else:
            if command:
                (output, ret) = pxssh.command('id')
                print(output)
                print("Return status: ", ret)
        pxssh.logout()
        del pxssh
        return True

    def create_group(self, group_name, external=False):
        """ Create external groups for Active Directory """
        if external:
            grp_add = "ipa group-add --desc='%s users external map' "\
                      "%s --external" % (group_name, group_name)
        else:
            grp_add = "ipa group-add --desc='%s users' %s" % (group_name,
                                                              group_name)
        cmd = self.multihost.run_command(grp_add, raiseonerr=False)
        if cmd.returncode != 0:
            raise SSSDException(cmd.stderr_text)

    def group_add_member(self, source_group, target_group, external=False):
        """ Make source group member of target group """
        if external:
            add_mem = "ipa -n group-add-member %s "\
                      "--external %s" % (target_group, source_group)
        else:
            add_mem = "ipa group-add-member %s --groups %s" % (target_group,
                                                               source_group)
        cmd = self.multihost.run_command(add_mem, raiseonerr=False)
        if cmd.returncode != 0:
            raise SSSDException(cmd.stderr_text)



""" Keytab Rotation Test cases

:requirement: IDM-SSSD-REQ: Active Directory host keytab renewal
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

from __future__ import print_function
import subprocess
import time
import pytest
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.samba import sambaTools


@pytest.mark.keytabrotation
class TestHostKeytabRotation(object):
    """ Keytab Rotation Test cases

    :setup:
      1. Configure RHEL client to join to Windows AD using
        realm
      2. set machine password age to 1 day in sssd.conf
    """

    @pytest.mark.tier2
    def test_001_rotation(self, multihost, keytab_sssd_conf):
        """
        :title: IDM-SSSD-TC: AD-Provider Keytab Rotation: Verify New entries in
         keytab should have new KVNO after keytab rotation
        :id: 26551713-7a82-489f-adc5-8543caa97dd2
        :steps:
          1. Set pwdLastSet attribute of computer account to 0
          2. Restart sssd
          3. sssd calls adcli and new host entries are added /etc/krb5.keytab
        :expectedresults:
          1. pwdLastSet Attribute should be 0
          2. Verify sssd restart successfully
          3. Verify /etc/krb5.keytab has new kvno apart from older entries
        """
        # Get current keytab entries
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.reset_machine_password()
        client_hostname = multihost.client[0].sys_hostname.split('.')[0]
        if len(client_hostname) > 15:
            client_hostname = client_hostname[:15]
        realm = multihost.ad[0].realm
        host_princ = 'HOST/%s@%s' % (client_hostname, realm)
        kvno_cmd = 'kvno %s' % (host_princ)
        cmd = multihost.client[0].run_command(kvno_cmd, raiseonerr=False)
        kvno = cmd.stdout_text.split('=')[1].strip()
        restart_sssd = 'systemctl restart sssd'
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            multihost.client[0].multihost.client[0].run_command(
                'journalctl -x -n 50 --no-pager -u sssd', raiseonerr=False)
            pytest.fail("Cannot restart sssd service")
        time.sleep(30)
        ls = 'ls -l /etc/krb5.keytab'
        cmd = multihost.client[0].run_command(ls, raiseonerr=False)
        klist_cmd = "klist -k /etc/krb5.keytab"
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        spn_list = [val.strip() for val in cmd.stdout_text.splitlines()]
        client_domain = multihost.client[0].sys_hostname.split('.')[0].upper()
        if len(client_domain) > 15:
            client_domain = client_domain[:15]
        old_kvno = int(kvno)
        new_kvno = int(kvno) + 1
        # older entry
        client_host_entry_1 = '{} {}/{}@{}'.format(old_kvno, 'host',
                                                   client_domain,
                                                   multihost.ad[0].realm)
        # new entry
        client_host_entry_2 = '{} {}/{}@{}'.format(new_kvno, 'host',
                                                   client_domain,
                                                   multihost.ad[0].realm)
        if client_host_entry_1 and client_host_entry_2 not in spn_list[3:]:
            pytest.fail("keytab rotation failed, host entries not rotated")

    @pytest.mark.tier2
    def test_002_updatedkeytab(self, multihost, keytab_sssd_conf):
        """
        :title: IDM-SSSD-TC: AD-Provider Keytab Rotation:
         Verify with updated Keytab Authentication using
         host credentials is successful
        :id: 21176af3-614e-417f-87b6-13d8dfe7f33a
        :steps:
          1. kinit using new HOST principal
          2. Do ldapsearch using GSSAPI bind
        :expectedresults:
          1. kinit should be successful
          2. ldapsearch should be successful
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.reset_machine_password()
        restart_sssd = 'systemctl restart sssd'
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            multihost.client[0].multihost.client[0].run_command(
                'journalctl -x -n 50 --no-pager -u sssd', raiseonerr=False)
            pytest.fail("Cannot restart sssd service")
        time.sleep(30)
        domain_basedn_entry = multihost.ad[0].domain_basedn_entry
        client_name = multihost.client[0].sys_hostname.strip().split('.')[0]
        if len(client_name) > 15:
            client_name = client_name[:15]
        client_host_keytab_entry = '{}$@{}'.format(client_name.upper(),
                                                   multihost.ad[0].realm)
        users_dn_entry = '{},{}'.format('CN=users', domain_basedn_entry)
        ad_hostname = multihost.ad[0].sys_hostname
        kinit_cmd = "kinit -k '%s'" % (client_host_keytab_entry)
        try:
            multihost.client[0].run_command(kinit_cmd)
        except subprocess.CalledProcessError:
            pytest.fail("kinit failed after keytab rotation")

        try:
            multihost.client[0].run_command(['klist'])
        except subprocess.CalledProcessError:
            pytest.fail("klist failed")

        ldap_cmd = "ldapsearch -H ldap://%s -Y GSSAPI -N -b %s "\
                   "'(&(objectclass=User) (sAMAccountName=Administrator))'" % (
                       ad_hostname, users_dn_entry)
        try:
            multihost.client[0].run_command(ldap_cmd)
        except subprocess.CalledProcessError:
            pytest.fail("ldapsearch using GSSAPI bind failed")

    @pytest.mark.tier2
    def test_003_delentry(self, multihost, keytab_sssd_conf):
        """
        :title: IDM-SSSD-TC: AD-Provider Keytab Rotation:
         Verify the oldest host principal entries are deleted
         after succesful machine password
        :id: 0812dcbf-5d58-4ff8-9891-789f1a5ce8d4
        :steps:
          1. Reset Machine password again by setting pwdlastSet attribute to 0
          2. Restart sssd service
          3. klist -k /etc/krb5.keytab
        :expectedresults:
          1. pwdLastSet attribute should be 0
          2. sssd service should be successfully restarted
          3. Verify adcli rotates keytab entries and the oldest keytab
             entry with KVNO 2 is deleted
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.reset_machine_password()
        domain_name = client.get_domain_section_name()
        klist_cmd = "klist -k /etc/krb5.keytab"
        client_hostname = multihost.client[0].sys_hostname.split('.')[0]
        if len(client_hostname) > 15:
            client_hostname = client_hostname[:15]
        realm = multihost.ad[0].realm
        host_princ = 'HOST/%s@%s' % (client_hostname, realm)
        kvno_cmd = 'kvno %s' % (host_princ)
        cmd = multihost.client[0].run_command(kvno_cmd, raiseonerr=False)
        kvno = cmd.stdout_text.split('=')[1].strip()
        remove_logs = "rm -f /var/log/sssd/sssd_%s.log" % (domain_name)
        multihost.client[0].run_command(remove_logs)
        restart_sssd = 'systemctl restart sssd'
        old_kvno = int(kvno)
        new_kvno1 = int(kvno) + 1
        new_kvno2 = new_kvno1 + 1
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            pytest.fail("Cannot restart sssd service")
        multihost.client[0].log.info("Sleep for 60 seconds")
        time.sleep(60)
        multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        client.reset_machine_password()
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            multihost.client[0].multihost.client[0].run_command(
                'journalctl -x -n 50 --no-pager -u sssd', raiseonerr=False)
            pytest.fail("Cannot restart sssd service")
        time.sleep(60)
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        spn_list = [val.strip() for val in cmd.stdout_text.splitlines()]
        client_domain = multihost.client[0].sys_hostname.split('.')[0].upper()
        if len(client_domain) > 15:
            client_domain = client_domain[:15]
        client_host_entry_old = '{} {}/{}@{}'.format(old_kvno, 'host',
                                                     client_domain,
                                                     multihost.ad[0].realm)
        client_host_entry1 = '{} {}/{}@{}'.format(new_kvno1, 'host',
                                                  client_domain,
                                                  multihost.ad[0].realm)
        client_host_entry2 = '{} {}/{}@{}'.format(new_kvno2, 'host',
                                                  client_domain,
                                                  multihost.ad[0].realm)
        if client_host_entry_old not in spn_list[3:]:
            assert client_host_entry1 and client_host_entry2 in spn_list[3:]
        else:
            pytest.fail("Client old host entry is not deleted from keytab")

    @pytest.mark.tier3
    def test_004_multiplespn(self, multihost, keytab_sssd_conf):
        """
        :title: IDM-SSSD-TC: AD-Provider Keytab Rotation:
         Add Multiple SPN(http,nfs) to the client host and
         verify all the SPN entries are rotated
        :id: a66c325f-09e2-4a81-8b76-b863dead7e92
        :steps:
          1. ADD HTTP SPN for client using net ads keytab cli
          2. ADD NFS SPN for client using net ads keytab cli
          3. Reset Machine password by setting pwdLastSet to 0
          4. Restart sssd
          5. klist -k /etc/krb5.keytab
        :expectedresults:
          1. klist -k /etc/krb5.keytab should HTTP entries
          2. klist -k /etc/krb5.keytab should NFS entries
          3. pwdLastSet attribute should be 0
          4. sssd service should be restarted successfully
          5. New HTTP and NFS entries with new kvno should be added to
             /etc/krb5.keytab
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.reset_machine_password()
        sambaclient = sambaTools(multihost.client[0], multihost.ad[0])
        sambaclient.smbadsconf()
        domain_name = client.get_domain_section_name()
        services_list = ['HTTP', 'NFS']
        client.add_service_principals(services_list)
        klist_cmd = "klist -k /etc/krb5.keytab"
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        realm = multihost.ad[0].realm
        hostname = multihost.client[0].sys_hostname
        remove_logs = "rm -f /var/log/sssd/sssd_%s.log" % (domain_name)
        multihost.client[0].run_command(remove_logs)
        restart_sssd = 'systemctl restart sssd'
        https_princ = 'HTTP/%s@%s' % (hostname, realm)
        kvno_cmd = 'kvno %s' % (https_princ)
        cmd = multihost.client[0].run_command(kvno_cmd, raiseonerr=False)
        kvno = cmd.stdout_text.split('=')[1].strip()
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            multihost.client[0].multihost.client[0].run_command(
                'journalctl -x -n 50 --no-pager -u sssd', raiseonerr=False)
            pytest.fail("Cannot restart sssd service")
        time.sleep(45)
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        spn_list = [val.strip() for val in cmd.stdout_text.splitlines()]
        new_kvno = int(kvno) + 1
        nfs_entry = '{} {}/{}@{}'.format(new_kvno, 'NFS', hostname, realm)
        http_entry = '{} {}/{}@{}'.format(new_kvno, 'HTTP', hostname, realm)
        assert nfs_entry and http_entry in spn_list[3:]
        client.remove_service_principals(services_list)
        remove_smb_conf = 'rm -f /etc/samba/smb.conf'
        multihost.client[0].run_command(remove_smb_conf, raiseonerr=False)

    @pytest.mark.tier3
    def test_005_deletespn(self, multihost, keytab_sssd_conf):
        """
        :title: IDM-SSSD-TC: AD-Provider Keytab Rotation:
         Removing SPN from AD and verify removed SPN entries
         are not renewed upon renewal
        :id: 6430387a-a715-44b4-81e3-7c012d887e00
        :steps:
          1. Delete HTTP SPN using setspn.exe  cli from AD
          2. Reset Machine password by setting pwdLastSet attribute to 0
          3. Restart sssd
          4. klist -k /etc/krb5.keytab
        :expectedresults:
          1. HTTP SPN should be deleted
          2. pwdLastSet attribute should be 0
          3. sssd service should be restarted successfuly
          4. Verify no new HTTP Entries with new KVNO are added in
             /etc/krb5.keytab
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        sambaclient = sambaTools(multihost.client[0], multihost.ad[0])
        sambaclient.smbadsconf()
        services_list = ['HTTP']
        client.add_service_principals(services_list)
        client.reset_machine_password()
        domain_name = client.get_domain_section_name()
        klist_cmd = "klist -k /etc/krb5.keytab"
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        realm = multihost.ad[0].realm
        hostname = multihost.client[0].sys_hostname
        remove_logs = "rm -f /var/log/sssd/sssd_%s.log" % (domain_name)
        multihost.client[0].run_command(remove_logs)
        restart_sssd = 'systemctl restart sssd'
        https_princ = 'HTTP/%s@%s' % (hostname, realm)
        kvno_cmd = 'kvno %s' % (https_princ)
        cmd = multihost.client[0].run_command(kvno_cmd, raiseonerr=False)
        kvno = cmd.stdout_text.split('=')[1].strip()
        client.remove_service_principals(services_list)
        try:
            multihost.client[0].run_command(restart_sssd)
        except subprocess.CalledProcessError:
            multihost.client[0].multihost.client[0].run_command(
                'journalctl -x -n 50 --no-pager -u sssd', raiseonerr=False)
            pytest.fail("Cannot restart sssd service")
        time.sleep(45)
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        spn_list = [val.strip() for val in cmd.stdout_text.splitlines()]
        new_kvno = int(kvno) + 1
        http_entry = '{} {}/{}@{}'.format(new_kvno, 'HTTP', hostname, realm)
        assert http_entry in spn_list[3:]
        cmd = multihost.client[0].run_command(klist_cmd, raiseonerr=False)
        remove_smb_conf = 'rm -f /etc/samba/smb.conf'
        multihost.client[0].run_command(remove_smb_conf, raiseonerr=False)



import pytest
from pytest_multihost import make_multihost_fixture

from .qe_class import QeConfig


@pytest.fixture(scope="session", autouse=True)
def session_multihost(request):
    # pylint: disable=no-member
    """Multihost plugin fixture for session scope"""
    if pytest.num_ad > 0:
        mhost = make_multihost_fixture(request, descriptions=[
            {
                'type': 'sssd',
                'hosts':
                {
                    'master': pytest.num_masters,
                    'atomic': pytest.num_atomic,
                    'replica': pytest.num_replicas,
                    'client': pytest.num_clients,
                    'other': pytest.num_others,
                }
            },
            {
                'type': 'ad',
                'hosts':
                {
                    'ad': pytest.num_ad,
                },
            },
        ], config_class=QeConfig,)
    else:
        mhost = make_multihost_fixture(request, descriptions=[
            {
                'type': 'sssd',
                'hosts':
                {
                    'master': pytest.num_masters,
                    'atomic': pytest.num_atomic,
                    'replica': pytest.num_replicas,
                    'client': pytest.num_clients,
                    'other': pytest.num_others,
                }
            },
        ], config_class=QeConfig,)
    mhost.domain = mhost.config.domains[0]
    mhost.master = mhost.domain.hosts_by_role('master')
    mhost.atomic = mhost.domain.hosts_by_role('atomic')
    mhost.replica = mhost.domain.hosts_by_role('replica')
    mhost.client = mhost.domain.hosts_by_role('client')
    mhost.others = mhost.domain.hosts_by_role('other')
    mhost.ad = []

    if pytest.num_ad > 0:
        mhost.ad = []
        for i in range(1, pytest.num_ad + 1):
            print(i)
            print(mhost.config.domains[i].hosts_by_role('ad'))
            mhost.ad.extend(mhost.config.domains[i].hosts_by_role('ad'))

    yield mhost


@pytest.fixture(scope='session', autouse=True)
def create_testdir(session_multihost, request):
    """
    Create test dir on the hosts and backup resolv.conf
    @param session_multihost: Multihost fixture
    @param request: Pytest request
    """
    print(f"Testdir is '{session_multihost.config.test_dir}'")
    test_dir = session_multihost.config.test_dir if \
        session_multihost.config.test_dir else '/root/multihost_tests'
    config_dir_cmd = f"mkdir -p {test_dir}"
    env_file_cmd = f"touch {test_dir}/env.sh"
    rm_config_cmd = f"rm -rf {test_dir}"
    ad_test_dir = '/home/Administrator'
    ad_config_dir_cmd = f"mkdir -p {ad_test_dir}"
    ad_env_file_cmd = f"touch {ad_test_dir}/env.sh"
    ad_rm_config_cmd = f"rm -rf {ad_test_dir}"
    bkup_resolv_conf = 'cp -a /etc/resolv.conf /etc/resolv.conf.orig'
    restore_resolv_conf = 'mv /etc/resolv.conf.orig /etc/resolv.conf'

    for machine in session_multihost.atomic + session_multihost.others +\
            session_multihost.replica:
        machine.run_command(config_dir_cmd)
        machine.run_command(env_file_cmd)

    for machine in session_multihost.client + session_multihost.master:
        machine.run_command(config_dir_cmd)
        machine.run_command(env_file_cmd)
        machine.run_command(bkup_resolv_conf)

    for machine in session_multihost.ad:
        machine.run_command(ad_config_dir_cmd)
        machine.run_command(ad_env_file_cmd)

    def remove_test_dir():
        for machine in session_multihost.client + session_multihost.master:
            machine.run_command(rm_config_cmd)
            machine.run_command("chattr -i /etc/resolv.conf", raiseonerr=False)
            machine.run_command(restore_resolv_conf, raiseonerr=False)

        for machine in session_multihost.atomic + session_multihost.others +\
                session_multihost.replica:
            machine.run_command(config_dir_cmd)

        for machine in session_multihost.ad:
            machine.run_command(ad_rm_config_cmd)

    request.addfinalizer(remove_test_dir)


@pytest.fixture(scope='session', autouse=True)
def disable_journald_rate_limit(session_multihost, request):
    """
    Update journald.conf
    To turn off any kind of rate limiting, set RateLimitIntervalSec value to 0.
    """
    cmd = session_multihost.client[0].run_command(
        'test -f /etc/systemd/journald.conf', raiseonerr=False)
    if cmd.returncode == 0:
        j_config = '/etc/systemd/journald.conf'
    else:
        j_config = '/usr/lib/systemd/journald.conf'

    bkup_cmd = f'cp -Zpf {j_config} /tmp/journald.conf.bkup'
    session_multihost.client[0].run_command(bkup_cmd, raiseonerr=False)
    up_ratelimit = 'RateLimitIntervalSec=0'
    journald_conf = session_multihost.client[0].get_file_contents(
        j_config)
    if isinstance(journald_conf, bytes):
        contents = journald_conf.decode('utf-8')
    else:
        contents = journald_conf
    contents = contents.replace(up_ratelimit, '') + up_ratelimit
    session_multihost.client[0].put_file_contents(j_config, contents)
    session_multihost.client[0].run_command(
        "systemctl restart systemd-journald", raiseonerr=False)

    def restore_journalsssd():
        """ Restore journalsssd.conf """
        bkup_cmd = f'cp -Zpf /tmp/journald.conf.bkup {j_config}'
        session_multihost.client[0].run_command(bkup_cmd)
    request.addfinalizer(restore_journalsssd)



"""Test cases for Multidomain

:requirement: multiple_domains
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import re
import datetime
import pytest
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client


@pytest.mark.usefixtures('posix_users_multidomain', 'sssdproxyldap',
                         'nslcd', 'template_sssdconf')
@pytest.mark.multidomain
class TestMultiDomain(object):
    @staticmethod
    @pytest.mark.tier1
    def test_ldap_referrals(multihost, multidomain_sssd):
        """
        :title: Ldap referrals feature of two ldap server with
            sssd option True and False.
        :id: ad937dc4-2fbc-11ee-97b0-845cf3eff344
        :setup:
            1. Set 'proxy_ldap2' domain in the SSSD configuration.
            2. The SSSD service is started on the client.
        :steps:
          1. Get the id of 'puser19'.
          2. Set sssd domain to ldap2.
          3. Try to get puser19 which only present master1 server.
          4. Set nsslapd-referral for master2 server.
          5. Set ldap_search_base for ldap2 domain matching with
            ldap_search_base of master1.
          6. Get the id of 'puser19'.
          7. Set 'ldap_referrals': False for ldap2 domain.
          8. Get the id of 'puser19'.
        :expectedresults:
          1. Should succeed
          2. Should succeed
          3. Should not succeed
          4. Should succeed
          5. Should succeed
          6. Should succeed
          7. Should succeed
          8. Should not succeed
        """
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('start')
        multihost.client[0].run_command("id puser19")
        client = sssdTools(multihost.client[0])
        domain_params = {'domains': 'ldap2'}
        client.sssd_conf('sssd', domain_params)
        multihost.client[0].service_sssd('restart')
        cmd0 = multihost.client[0].run_command("id puser19", raiseonerr=False)
        multihost.master[1].run_command(f'dsconf'
                                        f' -D "cn=Directory Manager"'
                                        f' -w Secret123 ldap://{multihost.master[1].sys_hostname} '
                                        f'config replace nsslapd-referral='
                                        f'"ldap://{multihost.master[0].sys_hostname}/"')
        params = {'ldap_search_base': 'dc=example0,dc=test',
                  'ldap_referrals': True}
        tools.sssd_conf('domain/ldap2', params)
        multihost.client[0].service_sssd('restart')
        cmd1 = multihost.client[0].run_command("id puser19")
        params = {'ldap_referrals': False}
        tools.sssd_conf('domain/ldap2', params)
        tools.clear_sssd_cache()
        cmd2 = multihost.client[0].run_command("id puser19", raiseonerr=False)
        multihost.master[1].run_command(f'dsconf -D '
                                        f'"cn=Directory Manager" -w Secret123 ldap://{multihost.master[1].sys_hostname} '
                                        f'config replace nsslapd-referral=""')
        assert cmd0.returncode != 0
        assert cmd2.returncode != 0
        assert cmd1

    @pytest.mark.tier2
    def test_0001_proxyldap2(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check config
         ldb filter users per domain for puser10 in proxy domain
        :id: aeed42e2-5b9b-4b04-b5f2-ea832250c38e
        """
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        proxy_params = {'filter_users': 'puser10'}
        tools.sssd_conf('domain/proxy', proxy_params)
        multihost.client[0].service_sssd('start')
        ldb_cmd = 'ldbsearch -H /var/lib/sss/db/config.ldb '\
                  '-b cn=proxy,cn=domain,cn=config'
        cmd = multihost.client[0].run_command(ldb_cmd, raiseonerr=False)
        if cmd.returncode == 0:
            find = re.compile(r'filter_users:\spuser10.*')
            if not find.search(cmd.stdout_text):
                pytest.fail('puser10 user not found in cache')

    @pytest.mark.tier2
    def test_0002_proxyldap2(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check config
         ldb filter users per domain for puser10 in ldap domain
        :id: 847490b5-0687-4443-bbf5-96c5a7dd2c9f
        """
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        ldap_params = {'filter_users': 'puser10'}
        tools.sssd_conf('domain/ldap1', ldap_params)
        multihost.client[0].service_sssd('start')
        ldb_cmd = 'ldbsearch -H /var/lib/sss/db/config.ldb '\
                  '-b cn=ldap1,cn=domain,cn=config'
        cmd = multihost.client[0].run_command(ldb_cmd, raiseonerr=False)
        if cmd.returncode == 0:
            find = re.compile(r'filter_users:\spuser10.*')
            if not find.search(cmd.stdout_text):
                pytest.fail('puser10 user not found in cache')

    @staticmethod
    @pytest.mark.tier2
    def test_0003_proxyldap2(multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain:
         checking lookup and authentication for proxy domain with filter_users
        :id: fb7d6f18-1c47-48f6-852f-824850b9f2d2
        """
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        proxy_params = {'filter_users': 'puser10',
                        'use_fully_qualified_names': 'True'}
        tools.sssd_conf('domain/proxy', proxy_params)
        multihost.client[0].service_sssd('start')
        users = ['puser10', 'puser10@proxy', 'puser11@proxy', 'quser10']
        for user in users:
            cmd = 'getent passwd %s' % user
            getent = multihost.client[0].run_command(cmd, raiseonerr=False)
            if 'puser10' in user:
                assert getent.returncode == 2
            else:
                assert getent.returncode == 0
        for user in ['puser11@proxy', 'quser10']:
            check_login_client(multihost, user, 'Secret123')

    @staticmethod
    @pytest.mark.tier2
    def test_0004_proxyldap2(multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain:
         checking lookup and authentication for ldap domain with filter_users
        :id: 7aa98ea5-1790-4407-b897-a15e8ebcf775
        """
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        ldap_params = {'filter_users': 'quser10',
                       'use_fully_qualified_names': 'True'}
        tools.sssd_conf('domain/ldap2', ldap_params)
        multihost.client[0].service_sssd('start')
        users = ['quser10', 'quser10@ldap2', 'quser11@ldap2', 'puser11']
        for user in users:
            cmd = 'getent passwd %s' % user
            getent = multihost.client[0].run_command(cmd, raiseonerr=False)
            if 'quser10' in user:
                assert getent.returncode == 2
            else:
                assert getent.returncode == 0
        for user in ['puser11', 'quser11@ldap2']:
            check_login_client(multihost, user, 'Secret123')

    @pytest.mark.tier2
    def test_0005_proxyldap2(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: proxy
         provider is not working with enumerate=true when trying to fetch
         all groups
        :id: d6e5958d-e719-4aab-b4a4-705f97191dfe
        """
        # Automation of BZ1665867
        multidomain_sssd(domains='proxy_ldap2')
        tools = sssdTools(multihost.client[0])
        timer = []
        for _ in range(5):
            multihost.client[0].service_sssd('stop')
            tools.remove_sss_cache('/var/lib/sss/db')
            multihost.client[0].service_sssd('start')
            t1 = datetime.datetime.now()
            starttime = t1.second
            grp_lk = 'getent group'
            multihost.client[0].run_command(grp_lk, raiseonerr=False)
            t2 = datetime.datetime.now()
            endtime = t2.second
            timer.append(endtime - starttime)
        print(timer)

    @pytest.mark.tier2
    def test_0006_filesproxy_ldapproxy(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: lookup
         and ldbsearch with filesproxy and ldapproxy domain
        :id: 01b34e45-d291-41f3-b54b-3364ce63e079
        """
        multidomain_sssd(domains='local_proxy')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        for idx in range(10):
            user = 'puser%d' % idx
            getent = 'getent passwd %s' % user
            cmd = multihost.client[0].run_command(getent, raiseonerr=False)
            assert cmd.returncode == 0
        provider = 'proxy'
        ldb = f'ldbsearch -H /var/lib/sss/db/config.ldb -b ' \
              f'cn={provider},cn=domain,cn=config'
        cmd = multihost.client[0].run_command(ldb, raiseonerr=False)
        if cmd.returncode == 0:
            checks = ['enumerate: True', f'id_provider: {provider}',
                      'max_id: [0-5]010', 'min_id: [0-5]000']
            for str1 in checks:
                find = re.compile(str1)
                result = find.search(cmd.stdout_text)
                assert result is not None

    @pytest.mark.tier2
    def test_0007_filesproxy_ldapproxy(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: lookup for
         localusers with domain filesproxy and ldapproxy
        :id: 33cd81a6-26a1-4235-8ef7-ebb52f6b6db2
        """
        multidomain_sssd(domains='local_proxy')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        for idx in range(10):
            user = 'puser%d' % idx
            getent = 'getent passwd %s' % user
            cmd = multihost.client[0].run_command(getent, raiseonerr=False)
            assert cmd.returncode == 0
        users = localusers
        for key, _ in users.items():
            l_user = 'getent passwd -s sss %s' % key
            cmd = multihost.client[0].run_command(l_user, raiseonerr=False)
            assert cmd.returncode == 0
            l_group = 'getent group -s sss %s' % key
            cmd = multihost.client[0].run_command(l_group, raiseonerr=False)
            assert cmd.returncode == 0

    @pytest.mark.tier2
    def test_0008_filesproxy_ldapproxy(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         modify LDAP domain User with domain filesproxy and ldapproxy
        :id: ecabce85-b620-45b9-a08a-d975952766a5
        """
        multidomain_sssd(domains='local_proxy')
        multihost.client[0].service_sssd('start')
        err_str = "usermod: user 'puser1' does not exist in /etc/passwd"
        usermod = 'usermod -g 5000 puser1'
        cmd = multihost.client[0].run_command(usermod, raiseonerr=False)
        assert cmd.returncode != 0
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0009_filesproxy_ldapproxy(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         delete LDAP domain User with domain filesproxy and ldapproxy
        :id: 327f75fc-d4f8-4d51-b140-117edf5f55eb
        """
        multidomain_sssd(domains='local_proxy')
        multihost.client[0].service_sssd('start')
        err_str = "userdel: user 'puser1' does not exist"
        userdel = 'userdel -r puser1'
        cmd = multihost.client[0].run_command(userdel, raiseonerr=False)
        assert cmd.returncode != 0
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0010_filesproxy_ldapproxy(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         modify group with domain filesproxy and ldapproxy
        :id: 662aaa9b-de83-4e43-8c9f-b8910e96644e
        """
        multidomain_sssd(domains='local_proxy')
        multihost.client[0].service_sssd('start')
        err_str = "groupmod: GID '5000' already exists"
        groupmod = 'groupmod -g 5000 pgroup0'
        cmd = multihost.client[0].run_command(groupmod, raiseonerr=False)
        assert cmd.returncode != 0
        print(cmd.stderr_text.strip().split('\n'))
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0011_filesproxy_ldapproxy(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain:  Attempt to
         delete group with domain filesproxy and ldapproxy
        :id: 28ecf278-b5bd-4d87-a63e-051010e55c5e
        """
        multidomain_sssd(domains='local_proxy')
        multihost.client[0].service_sssd('start')
        err_str = "groupdel: cannot remove the primary group of user 'puser0'"
        groupdel = 'groupdel pgroup0'
        cmd = multihost.client[0].run_command(groupdel, raiseonerr=False)
        assert cmd.returncode != 0
        print(cmd.stderr_text.strip().split('\n'))
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0012_filesldap(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: ldbsearch
         for local and ldap domain
        :id: bfb68697-9365-49ed-b277-1fef5cf0569b
        """
        multidomain_sssd(domains='local_ldap')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        for idx in range(10):
            user = 'quser%d' % idx
            getent = 'getent passwd %s' % user
            cmd = multihost.client[0].run_command(getent, raiseonerr=False)
            assert cmd.returncode == 0
        provider = 'ldap1'
        ldb = f'ldbsearch -H /var/lib/sss/db/config.ldb -b cn={provider},cn=domain,cn=config'
        cmd = multihost.client[0].run_command(ldb, raiseonerr=False)
        if cmd.returncode == 0:
            if provider == 'ldap1':
                provider = 'ldap'
            checks = ['enumerate: True', f'id_provider: {provider}',
                      'max_id: [0-5]010', 'min_id: [0-5]000']
            for str1 in checks:
                find = re.compile(str1)
                result = find.search(cmd.stdout_text)
                assert result is not None

    @pytest.mark.tier2
    def test_0013_proxyldap(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: User and
         group lookup for local and ldap domain
        :id: 50cc738c-d522-426f-a2cb-6e2ee37db86b
        """
        multidomain_sssd(domains='local_ldap')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        for idx in range(10):
            user = 'quser%d' % idx
            getent = 'getent passwd %s' % user
            cmd = multihost.client[0].run_command(getent, raiseonerr=False)
            assert cmd.returncode == 0
        users = localusers
        for key, _ in users.items():
            l_user = f'getent passwd {key}'
            cmd = multihost.client[0].run_command(l_user, raiseonerr=False)
            assert cmd.returncode == 0
            l_group = f'getent group {key}'
            cmd = multihost.client[0].run_command(l_group, raiseonerr=False)
            assert cmd.returncode == 0

    @pytest.mark.tier2
    def test_0014_filesldap(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         modify ldap user with local group id
        :id: 399362ee-ad63-4679-9409-6b15db3b6f63
        """
        multidomain_sssd(domains='local_ldap')
        multihost.client[0].service_sssd('start')
        err_str = "usermod: user 'quser1@ldap1' does not exist in /etc/passwd"
        usermod = 'usermod -g 5000 quser1@ldap1'
        cmd = multihost.client[0].run_command(usermod, raiseonerr=False)
        assert cmd.returncode != 0
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0015_filesldap(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         delete ldap user
        :id: 51db041b-880d-4b4b-a68c-fec92cdee291
        """
        multidomain_sssd(domains='local_ldap')
        multihost.client[0].service_sssd('start')
        err_str = "userdel: user 'quser1@ldap1' does not exist"
        userdel = 'userdel -r quser1@ldap1'
        cmd = multihost.client[0].run_command(userdel, raiseonerr=False)
        assert cmd.returncode != 0
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0016_filesldap(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         modify ldap group
        :id: cc3242cf-876e-4f58-bcd1-1532551bedf2
        """
        multidomain_sssd(domains='local_ldap')
        multihost.client[0].service_sssd('start')
        err_str = "groupmod: GID '5000' already exists"
        groupmod = 'groupmod -g 5000 qgroup0@ldap1'
        cmd = multihost.client[0].run_command(groupmod, raiseonerr=False)
        assert cmd.returncode != 0
        print(cmd.stderr_text.strip().split('\n'))
        assert err_str in cmd.stderr_text.strip().split('\n')

    @pytest.mark.tier2
    def test_0017_filesldap(self, multihost, multidomain_sssd, localusers):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Attempt to
         delete ldap domain group
        :id: 1a7a5a67-13a9-48d7-bf67-29238d5056fd
        """
        multidomain_sssd(domains='local_ldap')
        multihost.client[0].service_sssd('start')
        err_str1 = "groupdel: cannot remove entry 'qgroup0' from /etc/group"
        err_str2 = "groupdel: cannot remove the primary group of user 'quser0'"
        groupdel = 'groupdel qgroup0'
        cmd = multihost.client[0].run_command(groupdel, raiseonerr=False)
        assert cmd.returncode != 0
        print(cmd.stderr_text.strip().split('\n'))
        err_list = cmd.stderr_text.strip().split('\n')
        assert err_str1 in err_list or err_str2 in err_list

    @pytest.mark.tier2
    def test_0019_ldapldap(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: ldb search
         for users from two ldap domains
        :id: d00b674b-1f5a-4c72-ab5e-fe091a12c42c
        """
        multidomain_sssd(domains='ldap_ldap')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        for provider in ['ldap1', 'ldap2']:
            ldb = 'ldbsearch -H /var/lib/sss/db/config.ldb -b '\
                  'cn=%s,cn=domain,cn=config' % (provider)
            cmd = multihost.client[0].run_command(ldb, raiseonerr=False)
            if cmd.returncode == 0:
                if 'ldap' in provider:
                    provider = 'ldap'
                checks = ['enumerate: True', 'id_provider: %s' % provider,
                          'max_id: [0-5]020', 'min_uid: [0-5]000',
                          'cache_credentials: False',
                          'use_fully_qualified_names: True']
                for str1 in checks:
                    find = re.compile(r'%s' % str1)
                    result = find.search(cmd.stdout_text)
                    assert result is not None

    @pytest.mark.tier2
    def test_0020_ldapldap(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: checking
         users and groups lookup for two ldap domains
        :id: db99d818-ca66-45df-9b3e-2167cc50fab7
        """
        multidomain_sssd(domains='ldap_ldap')
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        suffix = ['p', 'q']
        for dom in range(2):
            for idx in range(20):
                user = '%suser%d@ldap%d' % (suffix[dom], idx, dom + 1)
                group = '%sgroup%d@ldap%d' % (suffix[dom], idx, dom + 1)
                lookup_u = 'getent passwd -s sss %s' % user
                lookup_g = 'getent group -s sss %s' % group
                cmd = multihost.client[0].run_command(lookup_u)
                assert cmd.returncode == 0
                cmd = multihost.client[0].run_command(lookup_g)
                assert cmd.returncode == 0

    @staticmethod
    @pytest.mark.tier2
    def test_0022_ldapldap(multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: User
         information not updated on login for secondary domains bz678593
        :id: df54756c-b141-4127-8e51-75ead63df10c
        """
        multidomain_sssd(domains='ldap_ldap')
        tools = sssdTools(multihost.client[0])
        ret = multihost.client[0].service_sssd('start')
        assert ret == 0
        suffix = ['p', 'q']
        for dom in range(2):
            for idx in range(5):
                user = '%suser%d@ldap%d' % (suffix[dom], idx, dom + 1)
                check_login_client(multihost, user, 'Secret123')
        pamlogfile = '/var/log/sssd/sssd_pam.log'
        find1 = re.compile(r'\[puser0\@ldap1\]')
        find2 = re.compile(r'\[quser0\@ldap2\]')
        log = multihost.client[0].get_file_contents(pamlogfile).decode('utf-8')
        assert find1.search(log) and find2.search(log)

    @staticmethod
    @pytest.mark.tier2
    def test_0023_ldapldap(multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Login time
         increases strongly while authenticating against a user from second
         domain
        :id: 29bdabfd-49ca-4040-a08d-b2b6adbde2cd
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=694905
         """
        multidomain_sssd(domains='ldap_ldap')
        client_tools = sssdTools(multihost.client[0])
        for idx in range(2):
            params = {'ldap_search_base': 'dc=example%d,dc=test' % idx,
                      'ldap_tls_reqcert': 'demand'}
            domain_section = 'domain/ldap%d' % (idx + 1)
            client_tools.sssd_conf(domain_section, params)
        domains = ['ldap1, ldap2', 'ldap2, ldap1']
        for domain in domains:
            sssd_params = {'domains': domain}
            client_tools.sssd_conf('sssd', sssd_params)
            multihost.client[0].service_sssd('stop')
            client_tools.remove_sss_cache('/var/lib/sss/db')
            ret = multihost.client[0].service_sssd('start')
            assert ret == 0
            suffix = ['p', 'q']
            timer = []
            for dom in range(2):
                t1 = datetime.datetime.now()
                starttime = t1.second
                print("start time = ", starttime)
                user = '%suser1@ldap%d' % (suffix[dom], dom + 1)
                print("user =", user)
                check_login_client(multihost, user, 'Secret123')
                t2 = datetime.datetime.now()
                endtime = t2.second
                print("end time = ", endtime)
                timer.append(endtime - starttime)
            print(timer)

    @pytest.mark.tier2
    def test_0024_bz1884196(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check
         lookup of user when enabled option is True in ldap1 domain
         and False in second ldap2 domain
        :id: a7ce3941-ba2c-407a-bed0-468aaab51fdb
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1884196
        """
        multidomain_sssd(domains='ldap_ldap')
        tools = sssdTools(multihost.client[0])
        ldap_params1 = {'enabled': 'True'}
        tools.sssd_conf('domain/ldap1', ldap_params1)
        ldap_params2 = {'enabled': 'False'}
        tools.sssd_conf('domain/ldap2', ldap_params2)
        tools.clear_sssd_cache()
        multihost.client[0].service_sssd('restart')
        for idx in range(10):
            user1 = 'puser%d@ldap%d' % (idx, 1)
            lookup_u1 = 'getent passwd %s' % user1
            cmd1 = multihost.client[0].run_command(lookup_u1)
            if cmd1.returncode == 0:
                status = 'PASS'
            else:
                status = 'FAIL'
        for idm in range(10):
            user2 = 'quser%d@ldap%d' % (idm, 2)
            try:
                lookup_u2 = 'getent passwd %s' % user2
                cmd2 = multihost.client[0].run_command(lookup_u2)
                print(cmd2.returncode)
                cmd2.returncode == 0
                status = 'FAIL'
            except Exception:
                status = 'PASS'
        assert status == 'PASS'

    @pytest.mark.tier2
    def test_0025_bz1884196(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check user
         when domains parameter has single domain but enabled True in both ldap
         domain
        :id: 33b9c044-3eef-472b-b9f9-d5de9f718c94
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1884196
        """
        multidomain_sssd(domains='ldap_ldap')
        tools = sssdTools(multihost.client[0])
        domain_params = {'domains': 'ldap1'}
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf('sssd', domain_params, action='update')
        ldap_params1 = {'enabled': 'True'}
        tools.sssd_conf('domain/ldap1', ldap_params1)
        ldap_params2 = {'enabled': 'True'}
        tools.sssd_conf('domain/ldap2', ldap_params2)
        tools.clear_sssd_cache()
        multihost.client[0].service_sssd('restart')
        suffix = ['p', 'q']
        for domain in range(2):
            for idx in range(10):
                user1 = '%suser%d@ldap%d' % (suffix[domain], idx, domain + 1)
                lookup_u1 = 'getent passwd %s' % user1
                cmd1 = multihost.client[0].run_command(lookup_u1)
                if cmd1.returncode == 0:
                    status = 'PASS'
                else:
                    status = 'FAIL'
        assert status == 'PASS'

    @pytest.mark.tier2
    def test_0026_bz1884196(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check
         enabled option with snippet file
        :id: 6fd1e9af-4039-49a1-bf4b-9925b042add5
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1884196
        """
        multidomain_sssd(domains='ldap_ldap')
        tools = sssdTools(multihost.client[0])
        domain_params = {'enable_files_domain': 'true'}
        tools.sssd_conf('sssd', domain_params, action='update')
        domain_params = {'domains': ''}
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf('sssd', domain_params, action='update')
        ldap_params1 = {'enabled': 'True'}
        tools.sssd_conf('domain/ldap1', ldap_params1)
        ldap_params2 = {'enabled': 'False'}
        tools.sssd_conf('domain/ldap2', ldap_params2)
        file_content = "[domain/ldap2]\nenabled = True"
        snippet_file = "/etc/sssd/conf.d/01_snippet.conf"
        multihost.client[0].put_file_contents(snippet_file, file_content)
        cmd_chmod = 'chmod 600 %s' % snippet_file
        multihost.client[0].run_command(cmd_chmod, raiseonerr=False)
        tools.clear_sssd_cache()
        multihost.client[0].service_sssd('restart')
        suffix = ['p', 'q']
        for domain in range(2):
            for idx in range(10):
                user1 = '%suser%d@ldap%d' % (suffix[domain], idx, domain + 1)
                lookup_u1 = 'getent passwd %s' % user1
                cmd1 = multihost.client[0].run_command(lookup_u1)
                if cmd1.returncode == 0:
                    status = 'PASS'
                else:
                    status = 'FAIL'
        delete_snip = 'rm -f /etc/sssd/conf.d/01_snippet.conf'
        multihost.client[0].run_command(delete_snip, raiseonerr=False)
        assert status == 'PASS'

    @pytest.mark.tier2
    def test_0027_bz1884196(self, multihost, multidomain_sssd):
        """
        :title: IDM-SSSD-TC: ldap_provider: test_for_multidomain: Check
         enabled option with snippet file and empty value of domains
         parameter in sssd section
        :id: bf65b8c7-f3ab-4ac1-a9a4-ad3377625a39
        :bugzilla:
         https://bugzilla.redhat.com/show_bug.cgi?id=1884196
        """
        multidomain_sssd(domains='ldap_ldap')
        tools = sssdTools(multihost.client[0])
        domain_params = {'enable_files_domain': 'true'}
        tools.sssd_conf('sssd', domain_params, action='update')
        domain_params = {'domains': ''}
        tools = sssdTools(multihost.client[0])
        tools.sssd_conf('sssd', domain_params, action='update')
        ldap_params1 = {'enabled': 'True'}
        tools.sssd_conf('domain/ldap1', ldap_params1)
        file_content = "[domain/ldap2]\nenabled = True"
        snippet_file = "/etc/sssd/conf.d/01_snippet.conf"
        multihost.client[0].put_file_contents(snippet_file, file_content)
        cmd_chmod = 'chmod 600 %s' % snippet_file
        multihost.client[0].run_command(cmd_chmod, raiseonerr=False)
        tools.clear_sssd_cache()
        multihost.client[0].service_sssd('restart')
        suffix = ['p', 'q']
        for domain in range(2):
            for idx in range(10):
                user1 = '%suser%d@ldap%d' % (suffix[domain], idx, domain + 1)
                lookup_u1 = 'getent passwd %s' % user1
                cmd1 = multihost.client[0].run_command(lookup_u1)
                if cmd1.returncode == 0:
                    status = 'PASS'
                else:
                    status = 'FAIL'
        delete_snip = 'rm -f /etc/sssd/conf.d/01_snippet.conf'
        multihost.client[0].run_command(delete_snip, raiseonerr=False)
        assert status == 'PASS'



""" AD idmap tests from bash
:requirement: IDM-SSSD-REQ: idmap
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
import pytest

from sssd.testlib.common.utils import sssdTools


@pytest.mark.usefixtures('joinad', 'create_idmap_users_groups')
@pytest.mark.tier2
@pytest.mark.idmap
class Testidmap(object):
    """ Test cases for idmap
    :setup:
        1. Join to AD using realm command.
        2. Create a non-POSIX user and a non-POSIX group
        3. Create a user and a group with posix attributes
    """
    @pytest.mark.converted('test_identity.py', 'test_identity__lookup_idmapping_of_posix_and_non_posix_user_and_group')
    @staticmethod
    def test_001_idmap_disable(multihost):
        """
        :title: with ldap provider idmapping is disabled
        :description: Disable idmapping for ldap provider and confirm the
         only the users, and group, with posix attributes defined in AD-server
         are returned
        :id: 989ec7eb-451c-49c4-9b09-e3ac005721d7
        :setup:
            1. Configure ldap_provider with idmapping disabled
        :steps:
            1. Fetch non-posix users and groups information
            2. Fetch posix users and groups information
            3. Log in as a posix-user
        :expectedresults:
            1. Non-posix user and group should not be returned
            2. POSIX user and group information should be returned
            3. POSIX user should be able to log in
        """
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        domain_name = client.get_domain_section_name()
        dom_section = f'domain/{domain_name}'
        params = {'id_provider': 'ldap',
                  'ldap_uri': f'ldap://{multihost.ad[0].hostname}',
                  'ldap_schema': 'ad',
                  'ldap_id_mapping': 'false',
                  'ldap_default_bind_dn': f'CN=administrator,CN=Users,{multihost.ad[0].domain_basedn_entry}',
                  'ldap_default_authtok': f'{multihost.ad[0].ssh_password}',
                  'ldap_default_authtok_type': 'password',
                  'debug_level': '0xFFF0',
                  'use_fully_qualified_names': 'false',
                  'ldap_tls_cacert': '/etc/openldap/certs/ad_cert.pem',
                  'ldap_tls_reqcert': 'demand',
                  'ldap_referrals': 'false'}
        client.sssd_conf(dom_section, params)
        client.clear_sssd_cache()
        cmd1 = multihost.client[0].run_command('getent passwd noposix_usr', raiseonerr=False)
        cmd2 = multihost.client[0].run_command('getent group noposix_grp', raiseonerr=False)
        cmd3 = multihost.client[0].run_command('getent passwd posix_usr', raiseonerr=False)
        cmd4 = multihost.client[0].run_command('getent group posix_grp', raiseonerr=False)
        client.restore_sssd_conf()
        assert client.auth_from_client('posix_usr', 'Secret123') == 3, 'ssh login failed for posix-user'
        assert cmd1.returncode == 2, 'non-posix user is returned by sssd, it should not be'
        assert cmd2.returncode == 2, 'non-posix group is returned by sssd, it should not be'
        assert cmd3.returncode == 0, 'posix-user is not returned by sssd'
        assert cmd4.returncode == 0, 'posix-group is not returned by sssd'



""" pexpect methods """

import sys
import pexpect
from pexpect import pxssh
from .exceptions import SSHLoginException
from .exceptions import OSException


class pexpect_ssh(object):
    """ pexpect methods """
    def __init__(self, hostname, username,
                 password, port=None,
                 encoding='utf-8', debug=False):
        """ Initilized defaults """
        self.hostname = hostname
        self.username = username
        self.password = password
        self.encoding = encoding
        if port is None:
            self.port = 22
        else:
            self.port = port
        self.ssh_options = {'StrictHostKeyChecking': 'no',
                            'UserKnownHostsFile': '/dev/null'}
        self.ssh = pxssh.pxssh(options=self.ssh_options)
        if debug:
            self.ssh.logfile = sys.stdout.buffer

    def login(self, login_timeout=10, auto_prompt_reset=True,
              sync_multiplier=1):
        """ login to host """
        self.PROMPT = r"\[PEXPECT\][\$\#] "
        try:
            self.ssh.login(self.hostname, self.username,
                           self.password, port=self.port,
                           login_timeout=login_timeout,
                           auto_prompt_reset=auto_prompt_reset,
                           sync_multiplier=sync_multiplier)
        except pexpect.pxssh.ExceptionPxssh:
            raise SSHLoginException("%s Failed to login" % self.username)

    def command(self, command, raiseonerr=False):
        """ Run Non interactive Commands """
        self.ssh.sendline(command)
        self.ssh.prompt()
        output_utf8 = self.ssh.before
        self.ssh.sendline("echo $?")
        self.ssh.prompt()
        returncode = self.ssh.before
        ret = returncode.decode('utf-8').splitlines()[1].strip('\n')
        output_str = output_utf8.decode('utf-8')
        if raiseonerr:
            if (int(ret)) != 0:
                raise OSException('Command failed with err: %s' % (output_str))
        return (output_str, ret)

    def expect_command(self, command, password, raiseonerr=False):
        """ Run interactive command prompting for password * """
        self.ssh.sendline(command)
        self.ssh.expect('Password.*.')
        self.ssh.sendline(password)
        cmd = self.ssh.expect(['Password incorrect .*.', r'[#\$] '])
        if cmd == 0:
            print("Password Incorrect")
        elif cmd == 1:
            print("Correct Password")
        output_utf8 = self.ssh.before
        self.ssh.sync_original_prompt()
        self.ssh.set_unique_prompt()
        self.ssh.sendline("echo $?")
        self.ssh.prompt()
        returncode = self.ssh.before.decode('utf-8')
        output_str = output_utf8.decode('utf-8')
        if raiseonerr:
            if (int(returncode)) != 0:
                raise OSException('Command failed with err: %s' % (output_str))
        return (output_str, returncode)

    def logout(self):
        """ Logout of ssh session """
        self.ssh.logout()



""" Automation of IPA subid feature bugs

:requirement: IDM-IPA-REQ: ipa subid range
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

import re
import tempfile
import pytest
from pexpect import pxssh


TEST_PASSWORD = "Secret123"
USER = 'admin'


def execute_cmd(multihost, command):
    """ Execute command on client """
    cmd = multihost.client[0].run_command(command)
    return cmd


def ipa_subid_find(multihost):
    """Grab and store information about admin subid ranges"""
    cmd = multihost.client[0].run_command(
        f'su - {USER} -c "ipa subid-find --owner {USER}"', raiseonerr=False)
    user_details = cmd.stdout_text.splitlines()

    global uid_start, uid_range, gid_start, gid_range
    uid_start = int(user_details[5].split(': ')[1].split('\n')[0])
    uid_range = int(user_details[6].split(': ')[1].split('\n')[0])
    gid_start = int(user_details[7].split(': ')[1].split('\n')[0])
    gid_range = int(user_details[8].split(': ')[1].split('\n')[0])


@pytest.mark.usefixtures('environment_setup',
                         'subid_generate',
                         'bkp_cnfig_for_subid_files')
@pytest.mark.tier1
class TestSubid(object):
    """
    This is for ipa bugs automation
    """

    @staticmethod
    def test_podmanmap_feature(multihost):
        """
        :Title: Podman supports subid ranges managed by FreeIPA
        :id: 0e86df9c-50f1-11ec-82f3-845cf3eff344
        :customerscenario: true
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1803943
        :steps:
            1. Test podman finds proper uid_map
            2. Test podman finds proper gid_map
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        ipa_subid_find(multihost)
        map1 = "/proc/self/uid_map"
        cmd = multihost.client[0].run_command(
            f'su - {USER} -c "podman unshare cat {map1}"', raiseonerr=False)
        actual_result = cmd.stdout_text.splitlines()

        assert str(uid_start) == actual_result[1].split()[1]
        assert str(uid_range) == actual_result[1].split()[2]
        map2 = "/proc/self/gid_map"
        cmd = multihost.client[0].run_command(
            f'su - {USER} -c "podman unshare cat {map2}"', raiseonerr=False)
        actual_result = cmd.stdout_text.splitlines()
        assert str(gid_start) == actual_result[1].split()[1]
        assert str(gid_range) == actual_result[1].split()[2]

    @staticmethod
    def test_subid_feature(multihost):
        """
        :Title: support subid ranges managed by FreeIPA
        :id: 50bcdc28-00c8-11ec-bef4-845cf3eff344
        :customerscenario: true
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1803943
        :steps:
            1. Test newuidmap command
            2. Test newgidmap command
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        ipa_subid_find(multihost)
        with tempfile.NamedTemporaryFile(mode='w') as tfile:
            tfile.write('#!/usr/bin/bash\n')
            tfile.write('whoami\n')
            tfile.write('unshare -U bash -c \'echo $$ >/tmp/unshare.pid;'
                        ' sleep 240; \' &\n')
            tfile.write('ps -ef | grep sleep | grep -v grep\n')
            tfile.write('MYPID="$(cat /tmp/unshare.pid)"\n')
            tfile.write(f'newuidmap $MYPID 0 {uid_start} 1\n')
            tfile.write('echo "uidmap_ $(cat /proc/$MYPID/uid_map) _uidmap"\n')
            tfile.write(f'newgidmap $MYPID 1000 {gid_start} 1\n')
            tfile.write('echo "gidmap_ $(cat /proc/$MYPID/gid_map) _gidmap"\n')
            tfile.flush()
            multihost.client[0].transport.put_file(tfile.name, '/tmp/maps.sh')
        multihost.client[0].run_command(
            'chmod ugo+x /tmp/maps.sh', raiseonerr=False)

        ssh = pxssh.pxssh(options={"StrictHostKeyChecking": "no",
                          "UserKnownHostsFile": "/dev/null"}, timeout=600)
        ssh.force_password = True
        try:
            ssh.login(multihost.client[0].ip, USER, TEST_PASSWORD)
            ssh.sendline('/tmp/maps.sh')
            ssh.prompt(timeout=600)
            ssh_output = str(ssh.before)
            ssh.logout()
        except pxssh.ExceptionPxssh as ex:
            pytest.fail(str(ex))
        multihost.client[0].run_command(
            f'echo "{ssh_output}"', raiseonerr=False)
        umap = re.search(f'uidmap_.*0.*{uid_start}.*1.*_uidmap', ssh_output)
        gmap = re.search(f'gidmap_.*1000.*{gid_start}.*1.*_gidmap', ssh_output)
        for file in ['maps.sh', 'unshare.pid']:
            multihost.client[0].run_command(f'rm -vf {file}')
        assert umap, "Expected uid map not found!"
        assert gmap, "Expected gid map not found!"
        assert "write to uid_map failed" not in ssh_output
        assert "write to gid_map failed" not in ssh_output

    @staticmethod
    def test_list_subid_ranges(multihost):
        """
        :Title: support subid ranges managed by FreeIPA
        :id: 4ab33f84-00c8-11ec-ad91-845cf3eff344
        :customerscenario: true
        :bugzilla: https://bugzilla.redhat.com/show_bug.cgi?id=1803943
        :steps:
            1. Test list_subid_ranges command
            2. Test list_subid_ranges -g command
        :expectedresults:
            1. Should succeed
            2. Should succeed
        """
        ipa_subid_find(multihost)
        multihost.client[0].run_command(
            f'su -l {USER} -c "whoami"', raiseonerr=False)
        cmd = multihost.client[0].run_command(f"cd /tmp/; "
                                              f"./list_subid_ranges "
                                              f"{USER}")
        assert str(USER) == cmd.stdout_text.split()[1]
        assert str(uid_start) == cmd.stdout_text.split()[2]
        assert str(uid_range) == cmd.stdout_text.split()[3]
        cmd = multihost.client[0].run_command(f"cd /tmp/;"
                                              f" ./list_subid_ranges"
                                              f" -g {USER}")
        assert str(USER) == cmd.stdout_text.split()[1]
        assert str(gid_start) == cmd.stdout_text.split()[2]
        assert str(gid_range) == cmd.stdout_text.split()[3]



""" Tests related to Caching of ssh keys

:requirement: ssh_authorizedkeys
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

import pytest
import re
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.ssh2_python import check_login_client


@pytest.mark.usefixtures('setup_sssd', 'create_posix_usersgroups',
                         'enable_ssh_schema', 'setup_sshd_authorized_keys',
                         'enable_ssh_responder')
@pytest.mark.ssh
class TestSSHkeys(object):
    """ Test ssh responder """

    @pytest.mark.tier1
    @pytest.mark.fips
    def test_0001_bz1137013(self, multihost, create_ssh_keys):
        """
        :title: ssh_authorizedkeys: OpenSSH LPK support
         by default bz1137013
        :id: 82c962b4-e740-4774-a3d1-b6e93a00bf26
        """
        tools = sssdTools(multihost.client[0])
        domain_name = tools.get_domain_section_name()
        user = 'foo1@%s' % domain_name
        check_login_client(multihost, user, 'Secret123')
        domain_log = '/var/log/sssd/sssd_%s.log' % domain_name
        log = multihost.client[0].get_file_contents(domain_log).decode('utf-8')
        msg = 'Adding sshPublicKey'
        find = re.compile(r'%s' % msg)
        assert find.search(log)



""" AD-Provider AD Schema tests ported from bash

:requirement: ad_schema
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
:caseautomation: Automated
:testtype: functional
"""

import random
import pytest


from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.utils import ADOperations

# Constants for id mapping
RANGE_MIN = 200000
RANGE_SIZE = 400000
RANGE_MAX = 2000200000


@pytest.fixture(scope="function", name="prepare_users")
def fixture_prepare_users(session_multihost, request):
    """Prepare users and groups"""
    uid = random.randint(9999, 99999)
    ad_op = ADOperations(session_multihost.ad[0])

    # Setup non-posix user 1
    ad_user_1 = 'testuser1%d' % uid
    ad_group_1 = 'testgroup1%d' % uid
    ad_op.create_ad_nonposix_user(ad_user_1)
    ad_op.create_ad_nonposix_group(ad_group_1)
    ad_op.add_user_member_of_group(ad_group_1, ad_user_1)

    # Add gecos to user 1
    usr = f"powershell.exe -inputformat none -noprofile 'Set-ADUser " \
        f"-Identity \"{ad_user_1}\" -Add @{{" \
        f"gecos = \"{ad_user_1}\";}}'"
    session_multihost.ad[0].run_command(usr, raiseonerr=False)

    # Set user primary group
    upg = f"powershell.exe -inputformat none -noprofile " \
        f"'Set-ADUserPrimaryGroup {ad_user_1} \'{ad_group_1}\''"
    res = session_multihost.ad[0].run_command(upg, raiseonerr=False)
    # Windows 2012R2 does not know Set-ADUserPrimaryGroup
    # This is a crude re-implementation
    if "'Set-ADUserPrimaryGroup' is not recognized" in res.stderr_text:
        info_cmd = f"powershell.exe -inputformat none -noprofile '" \
            f"write-host $(Get-ADGroup -Identity {ad_group_1}).SID'"
        cmd = session_multihost.ad[0].run_command(info_cmd, raiseonerr=False)
        group_id = cmd.stdout_text.strip().split('-')[-1]
        pgp_cmd = f"powershell.exe -inputformat none -noprofile Set-ADUser " \
            f"-Identity {ad_user_1} -Replace @{{'primaryGroupID' = '{group_id}'}}"
        session_multihost.ad[0].run_command(pgp_cmd, raiseonerr=False)

    # Setup posix user 2
    ad_user_2 = 'testuser2%d' % uid
    ad_group_2 = 'testgroup2%d' % uid
    ad_op.create_ad_unix_user_group(ad_user_2, ad_group_2)

    # Add gecos to user 2
    usr = f"powershell.exe -inputformat none -noprofile 'Set-ADUser " \
        f"-Identity \"{ad_user_2}\" -Add @{{gecos = \"{ad_user_2}\";}}'"
    session_multihost.ad[0].run_command(usr, raiseonerr=False)

    def remove_ad_user_groups():
        """ Remove windows AD users and groups"""
        ad_op.delete_ad_user_group(ad_user_1)
        ad_op.delete_ad_user_group(ad_group_1)
        ad_op.delete_ad_user_group(ad_user_2)
        ad_op.delete_ad_user_group(ad_group_2)

    request.addfinalizer(remove_ad_user_groups)
    return ad_user_1, ad_group_1, ad_user_2, ad_group_2


@pytest.mark.adschema
@pytest.mark.usefixtures("joinad")
class TestADSchema:
    """Automated Test Cases for AD Schema ported from bash"""

    @staticmethod
    @pytest.mark.tier1_3
    def test_0001_ad_schema_idmapping_true_user(multihost, prepare_users):
        """test_0001_ad_schema_idmapping_true_user

        :title: IDM-SSSD-TC: ad_provider: ad_schema: Compare with sysdb when
         idmapping is set to True for an user
        :id: fb75a597-7567-48c2-a786-74c6b4eeab37
        :setup:
          1. Configure ldap_idmap_range_size, ldap_id_mapping=True clear
         cache and restart sssd.
        :steps:
          1. Gather user information using getent passwd and run id command.
          2. Gather user information directly from AD (powershell).
          3. Gather user information from cache ldb.
          4. Compute user uid, gid.
          5. Compare gathered data and make sure that it is consistent.
        :expectedresults:
          1. User is found.
          2. Data is collected.
          3. Data is collected.
          4. Computed user uid, gid are matching the ones from getent.
          5. The content of data is consistent across the sources.
        :customerscenario: False
        """
        ad_realm = multihost.ad[0].domainname.upper()

        # Configure sssd
        multihost.client[0].service_sssd('stop')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_idmap_range_size': RANGE_SIZE,
        }
        client.sssd_conf(dom_section, sssd_params)
        client.clear_sssd_cache()

        # Get the non-posix user name from the fixture
        ad_user, _, _, _ = prepare_users

        # Gather the info about user from getent passwd
        getent_info = client.get_getent_passwd(f"{ad_user}@{ad_realm}")

        multihost.client[0].run_command(
            f'id {ad_user}@{ad_realm}', raiseonerr=False)

        ldb_info = client.dump_ldb(ad_user, ad_realm.lower())

        ad_op = ADOperations(multihost.ad[0])
        ad_info = ad_op.get_user_info(ad_user)

        uid, gid = ADOperations.compute_id_mapping(
            ad_info['objectSid'], int(ad_info['primaryGroupID']),
            range_min=RANGE_MIN, range_size=RANGE_SIZE, range_max=RANGE_MAX)

        ad_info['uidnumber_'], ad_info['gidnumber_'] = str(uid), str(gid)

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert ad_info['Name'] in getent_info['name']
        assert getent_info['uid'] == ad_info['uidnumber_']
        assert getent_info['gid'] == ad_info['gidnumber_']
        assert getent_info['gecos'] == ad_info['gecos']

        assert ad_info['Name'] == ldb_info['fullName']
        assert ad_info['SamAccountName'] in ldb_info['name']
        assert ad_info['uidnumber_'] == ldb_info['uidNumber']
        assert ad_info['gidnumber_'] == ldb_info['gidNumber']
        assert ad_info['gecos'] == ldb_info['gecos']
        assert ldb_info['originalMemberOf'].replace(" ", "") \
            in ad_info['MemberOf'].replace(" ", "")
        assert ad_info['userAccountControl'] == \
            ldb_info['adUserAccountControl']
        assert ad_info['objectSid'] == ldb_info['objectSIDString']

    @staticmethod
    @pytest.mark.tier1_3
    def test_0002_ad_schema_idmapping_true_group(multihost, prepare_users):
        """test_0002_ad_schema_idmapping_true_group

        :title: IDM-SSSD-TC: ad_provider: ad_schema: Compare with sysdb when
         idmapping is set to True for a group
        :id: 777bb5e3-6da5-495f-9098-754e483fa010
        :setup:
          1. Configure ldap_idmap_range_size, ldap_id_mapping=True clear
         cache and restart sssd.
        :steps:
          1. Gather group information using getent group.
          2. Gather group information directly from AD (powershell).
          3. Gather group information from cache ldb.
          4. Compute the gid for the group and compare with getent output.
          5. Compare gathered data and make sure that it is consistent.
        :expectedresults:
          1. Group is found.
          2. Data is collected.
          3. Data is collected.
          4. Computed gid is matching with the one from getent.
          5. The content of data is consistent across the sources.
        :customerscenario: False
        """
        ad_realm = multihost.ad[0].domainname.upper()

        # Configure sssd
        multihost.client[0].service_sssd('stop')
        client = sssdTools(multihost.client[0], multihost.ad[0])
        client.backup_sssd_conf()
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'True',
            'debug_level': '9',
            'id_provider': 'ad',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_idmap_range_size': RANGE_SIZE,
        }
        client.sssd_conf(dom_section, sssd_params)
        client.clear_sssd_cache()

        # Get the non-posix group name from the fixture
        _, ad_group, _, _ = prepare_users

        # Get info from getent
        getent_groupinfo = client.get_getent_group(f"{ad_group}@{ad_realm}")

        # Get group info from AD
        ad_op = ADOperations(multihost.ad[0])
        group_info = ad_op.get_group_info(ad_group)

        # Get group info from local cache ldb
        group_ldb_info = client.dump_ldb(ad_group, ad_realm.lower())

        # Compute group gid
        g_gid, _ = ADOperations.compute_id_mapping(
            group_info['objectSid'], 0,
            range_min=RANGE_MIN, range_size=RANGE_SIZE, range_max=RANGE_MAX)
        group_info['gidnumber_'] = str(g_gid)

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert group_info['Name'] in getent_groupinfo['name']
        assert group_info['gidnumber_'] == getent_groupinfo['gid']
        assert group_info['Name'] in group_ldb_info['name']
        assert group_info['gidnumber_'] == group_ldb_info['gidNumber']
        assert group_info['objectSid'] == group_ldb_info['objectSIDString']

        # Windows 2012 has a different format is different
        if 'member' in group_info:
            assert getent_groupinfo['users'].split("@")[0] in \
                   group_info['member']
            assert group_ldb_info['orig_member'] in group_info['member']
        elif 'members' in group_info:
            assert getent_groupinfo['users'].split("@")[0] in \
                   group_info['members']
            assert group_ldb_info['orig_member'] in group_info['members']

    @staticmethod
    @pytest.mark.tier1_3
    def test_0003_ad_schema_idmapping_false_user(multihost, prepare_users):
        """
        :title: IDM-SSSD-TC: ad_provider: ad_schema: Compare with sysdb when
         idmapping is set to False an user
        :id: bef6b2d1-656c-48f9-b6ff-2153d10c3556
        :setup:
          1. Configure ldap_idmap_range_size, ldap_id_mapping=False clear
          cache and restart sssd.
        :steps:
          1. Gather user information using getent passwd and run id command.
          2. Gather user information directly from AD (powershell).
          3. Gather user information from cache ldb.
          4. Compare gathered data and make sure that it is consistent.
        :expectedresults:
          1. User is found.
          2. Data is collected.
          3. Data is collected.
          4. The content of data is consistent across the sources.
        :customerscenario: False
        """
        ad_realm = multihost.ad[0].domainname.upper()
        client = sssdTools(multihost.client[0], multihost.ad[0])

        # Backup the config because with broken config we can't leave ad
        client.backup_sssd_conf()

        # Configure sssd to ad_domain = junk
        multihost.client[0].service_sssd('stop')
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'False',
            'debug_level': '9',
            'id_provider': 'ad',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_idmap_range_size': RANGE_SIZE,
        }
        client.sssd_conf(dom_section, sssd_params)
        client.clear_sssd_cache()

        # Get the posix user name from the fixture
        _, _, ad_user, _ = prepare_users

        # Gather the info about user from getent passwd
        getent_info = client.get_getent_passwd(f"{ad_user}@{ad_realm}")

        multihost.client[0].run_command(
            f'id {ad_user}@{ad_realm}', raiseonerr=False)

        ldb_info = client.dump_ldb(ad_user, ad_realm.lower())

        ad_op = ADOperations(multihost.ad[0])
        ad_info = ad_op.get_user_info(ad_user)

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert ad_info['Name'] in getent_info['name']
        assert getent_info['uid'] == ad_info['uidNumber']
        assert getent_info['gid'] == ad_info['gidNumber']
        assert getent_info['gecos'] == ad_info['gecos']
        assert getent_info['home'] == ad_info['unixHomeDirectory']
        assert getent_info['shell'] == ad_info['loginShell']

        assert ad_info['Name'] == ldb_info['fullName']
        assert ad_info['SamAccountName'] in ldb_info['name']
        assert ad_info['uidNumber'] == ldb_info['uidNumber']
        assert ad_info['gidNumber'] == ldb_info['gidNumber']
        assert ad_info['loginShell'] == ldb_info['loginShell']
        assert ad_info['uSNChanged'] == ldb_info['entryUSN']
        assert ad_info['gecos'] == ldb_info['gecos']
        assert ad_info['unixHomeDirectory'] == ldb_info['homeDirectory']
        assert ad_info['accountExpires'] == ldb_info['adAccountExpires']
        assert ldb_info['originalMemberOf'].replace(" ", "") in \
            ad_info['MemberOf'].replace(" ", "")
        assert ad_info['userAccountControl'] == \
            ldb_info['adUserAccountControl']
        assert ad_info['objectSid'] == ldb_info['objectSIDString']

    @staticmethod
    @pytest.mark.tier1_3
    def test_0004_ad_schema_idmapping_false_group(multihost, prepare_users):
        """
        :title: IDM-SSSD-TC: ad_provider: ad_schema: Compare with sysdb when
         idmapping is set to False for a group
        :id: b1856f79-cbf8-4dd5-a1bd-a3761c1a4432
        :setup:
          1. Configure ldap_idmap_range_size, ldap_id_mapping=False clear
          cache and restart sssd.
        :steps:
          1. Gather group information using getent group.
          2. Gather group information directly from AD (powershell).
          3. Gather group information from cache ldb.
          4. Compare gathered data and make sure that it is consistent.
        :expectedresults:
          1. Group is found.
          2. Data is collected.
          3. Data is collected.
          4. The content of data is consistent across the sources.
        :customerscenario: False
        """
        ad_realm = multihost.ad[0].domainname.upper()
        client = sssdTools(multihost.client[0], multihost.ad[0])

        # Backup the config because with broken config we can't leave ad
        client.backup_sssd_conf()

        # Configure sssd to ad_domain = junk
        multihost.client[0].service_sssd('stop')
        dom_section = f'domain/{client.get_domain_section_name()}'
        sssd_params = {
            'ldap_id_mapping': 'False',
            'debug_level': '9',
            'id_provider': 'ad',
            'ad_domain': multihost.ad[0].domainname.lower(),
            'ad_server': multihost.ad[0].hostname,
            'ldap_idmap_range_size': RANGE_SIZE,
        }
        client.sssd_conf(dom_section, sssd_params)
        client.clear_sssd_cache()

        # Get the posix group name from the fixture
        _, _, _, ad_group = prepare_users

        # Get info from getent
        getent_groupinfo = client.get_getent_group(f"{ad_group}@{ad_realm}")

        # Get group info from AD
        ad_op = ADOperations(multihost.ad[0])
        group_info = ad_op.get_group_info(ad_group)

        # Get group info from local cache ldb
        group_ldb_info = client.dump_ldb(ad_group, ad_realm.lower())

        client.restore_sssd_conf()
        client.clear_sssd_cache()

        # Evaluate test results
        assert group_info['Name'] in getent_groupinfo['name']
        assert group_info['gidNumber'] == getent_groupinfo['gid']
        assert group_info['Name'] in group_ldb_info['name']
        assert group_info['gidNumber'] == group_ldb_info['gidNumber']
        assert group_info['objectSid'] == group_ldb_info['objectSIDString']
        assert group_info['uSNChanged'] == group_ldb_info['entryUSN']
        # Windows 2012 has a different format is different
        if 'member' in group_info:
            assert getent_groupinfo['users'].split("@")[0] in \
                   group_info['member']
            assert group_ldb_info['orig_member'] in group_info['member']
        elif 'members' in group_info:
            assert getent_groupinfo['users'].split("@")[0] in \
                   group_info['members']
            assert group_ldb_info['orig_member'] in group_info['members']



#
# Various functions
#
# Copyright (c) 2015 Red Hat, Inc.
# Author: Nikolai Kondrashov <Nikolai.Kondrashov@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import os
import sys
import subprocess
import config
import shutil

UNINDENT_RE = re.compile("^ +", re.MULTILINE)


def unindent(text):
    """
        Unindent text by removing at most the number of spaces present in
        the first non-empty line from the beginning of every line.
    """
    indent_ref = [0]

    def replace(match):
        if indent_ref[0] == 0:
            indent_ref[0] = len(match.group())
        return match.group()[indent_ref[0]:]
    return UNINDENT_RE.sub(replace, text)


def run_shell():
    """
        Execute an interactive shell under "screen", preserving environment.
        For use as a breakpoint for debugging.
    """
    my_env = os.environ.copy()
    my_env["ROOT_DIR"] = config.PREFIX

    # screen filter out LD_* evniroment varibles.
    # Back-up them and set them later in screenrc
    my_env["_LD_LIBRARY_PATH"] = os.getenv("LD_LIBRARY_PATH", "")
    my_env["_LD_PRELOAD"] = os.getenv("LD_PRELOAD", "")

    subprocess.call([
        "screen", "-DAm", "-S", "sssd_cwrap_session", "-c",
        ".config/screenrc"],
        env=my_env
    )


def first_dir(*args):
    """Return first argument that points to an existing directory."""
    for arg in args:
        if os.path.isdir(arg):
            return arg


def backup_envvar_file(name):
    path = os.environ[name]
    backup_path = path + ".bak"
    shutil.copyfile(path, backup_path)
    return path


def restore_envvar_file(name):
    path = os.environ[name]
    backup_path = path + ".bak"
    os.rename(backup_path, path)


def get_call_output(cmd, stderr_output=subprocess.PIPE, check=False, custom_env=None):
    """
        Executes the provided command.
        When check is set to True, this function will throw an exception
        if the command returns with a non-zero value.
    """

    if (sys.version_info.major < 3
            or (sys.version_info.major == 3 and sys.version_info.minor < 7)):
        try:
            output = subprocess.check_output(cmd, universal_newlines=True,
                                             stderr=stderr_output, env=custom_env)
        except subprocess.CalledProcessError as err:
            if (not check):
                output = err.output
            else:
                raise err
        return output

    process = subprocess.run(cmd, check=check, text=True,
                             stdout=subprocess.PIPE, stderr=stderr_output,
                             env=custom_env)
    return process.stdout



"""Automation for ldap_library_debug_level

:requirement: IDM-SSSD-REQ : LDAP Provider
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import re
import pytest
from sssd.testlib.common.utils import sssdTools
from constants import ds_instance_name


@pytest.mark.usefixtures('setup_sssd', 'create_posix_usersgroups')
@pytest.mark.ldaplibdebuglevel
class TestLdapLibDebugLevel(object):
    """ Test ldap_library_debug_level option"""

    @pytest.mark.tier1_2
    def test_0001_bz1884207(self, multihost, backupsssdconf):
        """
        :title: ldap_library_debug_level: Check ldap_library_debug_level
         option with config-check
        :id: b753a633-53ca-42ba-974e-cab6bfad17d2
        """
        section = "domain/%s" % ds_instance_name
        tools = sssdTools(multihost.client[0])
        domain_params = {'ldap_library_debug_level': '0'}
        tools.sssd_conf(section, domain_params)
        multihost.client[0].service_sssd('restart')
        sssctl_cmd = 'sssctl config-check'
        cmd = multihost.client[0].run_command(sssctl_cmd, raiseonerr=False)
        assert cmd.returncode == 0

    @pytest.mark.tier1_2
    def test_0002_bz1884207(self, multihost, backupsssdconf):
        """
        :title: ldap_library_debug_level: Set ldap_library_debug_level to
         zero and check corresponding logs
        :id: 6cf52e0f-594b-42b7-a933-6bf4257603c9
        """
        section = "domain/%s" % ds_instance_name
        tools = sssdTools(multihost.client[0])
        domain_params = {'ldap_library_debug_level': '0'}
        tools.sssd_conf(section, domain_params)
        tools.clear_sssd_cache()
        logfile = '/var/log/sssd/sssd_%s.log' % ds_instance_name
        log_str = multihost.client[0].get_file_contents(logfile)
        find = re.compile(r'libldap')
        assert not find.search(log_str.decode())

    @pytest.mark.tier1_2
    def test_0003_bz1884207(self, multihost, backupsssdconf):
        """
        :title: ldap_library_debug_level: Set ldap_library_debug_level to
         two and check corresponding logs
        :id: 97e03505-d5f3-45df-b6ed-f7ede1106f07
        """
        section = "domain/%s" % ds_instance_name
        tools = sssdTools(multihost.client[0])
        domain_params = {'ldap_library_debug_level': '2'}
        tools.sssd_conf(section, domain_params)
        tools.clear_sssd_cache()
        logfile = '/var/log/sssd/sssd_%s.log' % ds_instance_name
        log_str1 = multihost.client[0].get_file_contents(logfile)
        find = re.compile(r'libldap')
        assert find.search(log_str1.decode(errors='ignore'))



""" Automation of auto private groups

:requirement: IDM-SSSD-REQ: SSSD can automatically create\
 user private groups for users
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
from __future__ import print_function
import pytest
from sssd.testlib.common.utils import sssdTools
from constants import ds_instance_name


@pytest.mark.usefixtures('setup_sssd',
                         'create_posix_usersgroups',
                         'create_posix_usersgroups_autoprivategroups')
@pytest.mark.autoprivategroup
class TestAutoPrivateGroups(object):
    @pytest.mark.converted('test_identity.py', 'test_identity__lookup_when_private_groups_set_to_hybrid')
    @pytest.mark.tier1
    def test_0001_bz1695577(self, multihost, backupsssdconf):
        """
        :title: user_private_group: auto_private_group set to hybrid
         uid equals to gid and group exists
        :id: 19675eaa-d459-40da-985c-b8e89f8bea40
        """
        multihost.client[0].service_sssd('stop')
        tools = sssdTools(multihost.client[0])
        apg = {'auto_private_groups': 'hybrid'}
        tools.sssd_conf('domain/%s' % (ds_instance_name), apg)
        tools.remove_sss_cache('/var/lib/sss/db')
        multihost.client[0].service_sssd('start')
        for i in range(9):
            lkup = 'id foobar%d@%s' % (i, ds_instance_name)
            cmd = multihost.client[0].run_command(lkup, raiseonerr=False)
            lkup = 'getent group foobar%d@%s' % (i, ds_instance_name)
            cmd = multihost.client[0].run_command(lkup, raiseonerr=False)
            lkup = 'getent passwd foobar%d@%s' % (i, ds_instance_name)
            cmd = multihost.client[0].run_command(lkup, raiseonerr=False)
            output = cmd.stdout_text.split(':')
            assert int(output[2]) == int(output[3])

    @pytest.mark.converted('test_identity.py', 'test_identity__lookup_when_private_groups_set_to_hybrid')
    @pytest.mark.tier1
    def test_0002_bz1695577(self, multihost, backupsssdconf):
        """
        :title: user_private_group: auto_private_group set to hybrid
         uid does not equals to gid and group does exists
        :id: bd7cda5a-49d9-4ca7-8dc6-c0f6c39b494a
        """
        multihost.client[0].service_sssd('stop')
        tools = sssdTools(multihost.client[0])
        apg = {'auto_private_groups': 'hybrid'}
        tools.sssd_conf('domain/%s' % (ds_instance_name), apg)
        tools.remove_sss_cache('/var/lib/sss/db')
        multihost.client[0].service_sssd('start')
        lkup = 'getent passwd foobar11@%s' % (ds_instance_name)
        cmd = multihost.client[0].run_command(lkup, raiseonerr=False)
        output = cmd.stdout_text.split(':')
        assert int(output[2]) != int(output[3])



""" Automation for sssd failover

:requirement: IDM-SSSD-REQ : Failover
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""
import pytest
from sssd.testlib.common.utils import sssdTools
from constants import ds_instance_name
from sssd.testlib.common.ssh2_python import check_login_client_bool


@pytest.mark.usefixtures('multipleds_failover',
                         'create_posix_usersgroups_failover',
                         'setup_sssd_failover', )
@pytest.mark.failover
class TestFailover(object):
    """ Bug 1283798 failover automation
    :setup:
      1. Configure Directory servers on 2 Hosts (ldap1, ldap2)
      with TLS
      2. Configure sssd.conf on client with auth_provider: ldap
      3. specify ldaps in ldap_uri pointing to 2 directory servers
      example: ldap_uri: ldaps://ldap1, ldaps://ldap2
    """
    @staticmethod
    @pytest.mark.tier2
    def test_0001_getent(multihost):
        """
        :title: failover: Verify users can be queried from
         second directory server when first directory server is down
        :id: 0d145340-e147-4da7-acd0-f1c29891c397
        """
        # query ldap users when both ldaps servers are working
        user = 'foo0@%s' % ds_instance_name
        getent = 'getent passwd %s' % user
        cmd = multihost.client[0].run_command(getent)
        assert cmd.returncode == 0
        tools = sssdTools(multihost.client[0])
        # stop first directory server instance_name
        stop_ds1 = 'systemctl stop dirsrv@example'
        cmd = multihost.master[0].run_command(stop_ds1, raiseonerr=False)
        assert cmd.returncode == 0
        # query the new  user foo1
        user = 'foo1@%s' % ds_instance_name
        getent = 'getent passwd %s' % user
        cmd = multihost.client[0].run_command(getent)
        assert cmd.returncode == 0
        # clear the cache and query foo1 user again
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        multihost.client[0].service_sssd('start')
        cmd = multihost.client[0].run_command(getent)
        assert cmd.returncode == 0
        # start the first directory server
        start_ds1 = 'systemctl start dirsrv@example'
        cmd = multihost.master[0].run_command(start_ds1, raiseonerr=False)
        assert cmd.returncode == 0

    @staticmethod
    @pytest.mark.tier2
    def test_0002_login(multihost):
        """
        :title: failover: Verify users can login when the first
         ldap server is down
        :id: 9c0e0448-3fc2-44c7-96f8-9b8b44fa5cba
        """
        user = 'foo2@%s' % ds_instance_name
        stop_ds1 = 'systemctl stop dirsrv@example'
        cmd = multihost.master[0].run_command(stop_ds1, raiseonerr=False)
        assert cmd.returncode == 0
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        multihost.client[0].service_sssd('start')
        # login as user
        ssh = check_login_client_bool(multihost, user, 'Secret123')
        start_ds1 = 'systemctl start dirsrv@example'
        cmd = multihost.master[0].run_command(start_ds1, raiseonerr=False)
        assert ssh, f'{user} is not able to login.'
        assert cmd.returncode == 0

    @staticmethod
    @pytest.mark.tier2
    def test_0003_stopsecondds(multihost):
        """
        :title: failover: Stop second ldap server and verify
         users are able to login from first ldap server
        :id: cf15aea7-a626-4ed2-a205-9180ddfe29b2
        """
        stop_ds2 = 'systemctl stop dirsrv@example'
        cmd = multihost.master[1].run_command(stop_ds2, raiseonerr=False)
        assert cmd.returncode == 0
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        multihost.client[0].service_sssd('start')
        user = 'foo3@%s' % ds_instance_name
        # login as user
        ssh = check_login_client_bool(multihost, user, 'Secret123')
        start_ds1 = 'systemctl start dirsrv@example'
        cmd = multihost.master[0].run_command(start_ds1, raiseonerr=False)
        assert ssh, f'{user} is not able to login.'
        assert cmd.returncode == 0



""" Automation of sssctl suite with ldap and krb5 provider

:requirement: IDM-SSSD-REQ: Status utility
:casecomponent: sssd
:subsystemteam: sst_idm_sssd
:upstream: yes
:status: approved
"""

from __future__ import print_function
import re
import pytest
import time
from sssd.testlib.common.utils import sssdTools
from sssd.testlib.common.helper_functions import configure_proxy
from constants import ds_instance_name


def client_version(multihost):
    if [int(s) for s in multihost.client[0].distro if s.isdigit()][0] >= 9:
        return True


@pytest.mark.usefixtures('setup_sssd', 'create_posix_usersgroups',
                         'sssdproxyldap')
@pytest.mark.sssctl
class Testsssctl(object):
    """ This is test case class for sssctl suite """
    @pytest.mark.tier1_2
    def test_0001_bz1638295(self, multihost, backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to root and ldap user
        :id: aaa3c20e-176c-404a-8845-37a524222b14
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].run_command("useradd foo1", raiseonerr=False)
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        tools.sssd_conf("sssd", {'services': 'nss, pam, ifp'}, action='update')
        domain_params = {'allowed_uids': 'root, foo1',
                         'user_attributes': '+mail, -gecos'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks foo1@%s' % ds_instance_name
        cmd = multihost.client[0].run_command(sssctl_cmd, raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'gecos: not set',
                  'mail: foo1@example.test']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None
        multihost.client[0].run_command("userdel -rf foo1", raiseonerr=False)

    @pytest.mark.tier1_2
    def test_0002_bz1638295(self, multihost, backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to 0 and ldap user's uid
        :id: 8f7f57ea-bc3d-4b1b-989f-b3b4d3c88a14
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        tools.sssd_conf("sssd", {'services': 'nss, pam, ifp'}, action='update')
        domain_params = {'allowed_uids': '0, 14583101',
                         'user_attributes': '+mail, -gecos'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks foo1@%s' % ds_instance_name
        cmd = multihost.client[0].run_command(sssctl_cmd, raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'gecos: not set',
                  'mail: foo1@example.test']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None

    @pytest.mark.tier1_2
    def test_0003_bz1638295(self, multihost,
                            localusers,
                            backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to root and localuser
        :id: 4aafb8f5-3fad-46b3-91a3-e431e872a4af
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        configure_proxy(multihost)
        domain_params = {'allowed_uids': 'root, user5000',
                         'user_attributes': '-name, -uidNumber'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks user5000'
        cmd = multihost.client[0].run_command(sssctl_cmd,
                                              raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'name: not set',
                  'uidNumber: not set']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None

    @pytest.mark.tier1_2
    def test_0004_bz1638295(self, multihost,
                            localusers,
                            backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to root's and
         localuser's uid
        :id: e46353b3-904f-4226-b6fa-4444c67861b0
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        configure_proxy(multihost)
        domain_params = {'allowed_uids': '0, 5000',
                         'user_attributes': '-name, -uidNumber'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        cmd_id = multihost.client[0].run_command("id user5000", raiseonerr=False)
        if cmd_id.returncode != 0:
            multihost.client[0].run_command("useradd -u 5000 user5000")
            multihost.client[0].run_command("passwd --stdin user5000", stdin_text='Secret123')

        sssctl_cmd = 'sssctl user-checks user5000'
        cmd = multihost.client[0].run_command(sssctl_cmd,
                                              raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'name: not set',
                  'uidNumber: not set']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None

    @pytest.mark.tier1_2
    def test_0005_bz1638295(self, multihost,
                            localusers,
                            backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to localuser's uids
        :id: 96b47d83-e123-42fc-8952-381975e95f9b
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        configure_proxy(multihost)
        domain_params = {'allowed_uids': '5000',
                         'user_attributes': '-name, -uidNumber'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks user5000'
        cmd = multihost.client[0].run_command(sssctl_cmd,
                                              raiseonerr=False)
        assert cmd.returncode == 0
        find = re.compile(r'InfoPipe\sUser\slookup\swith\s.user5000.\sfailed')
        result = find.search(cmd.stderr_text)
        assert result is not None

    @pytest.mark.tier1_2
    def test_0006_bz1638295(self, multihost, backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to 0 and ldap user
        :id: 78cb0086-0107-49e1-93ca-efcc4a414aad
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].run_command("useradd foo1", raiseonerr=False)
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        tools.sssd_conf("sssd", {'services': 'nss, pam, ifp'}, action='update')
        domain_params = {'allowed_uids': '0, foo1',
                         'user_attributes': '+mail, -gecos'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks foo1@%s' % ds_instance_name
        cmd = multihost.client[0].run_command(sssctl_cmd, raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'gecos: not set', 'mail: foo1@example.test']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None
        multihost.client[0].run_command("userdel -rf foo1", raiseonerr=False)

    @pytest.mark.tier1_2
    def test_0007_bz1638295(self, multihost,
                            localusers,
                            backupsssdconf):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl user-checks does not show custom
         IFP user_attributes with allowed_uids equal to root and localuser's
         uid
        :id: 90f6f768-d006-4d8a-a289-5be50205aa0e
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].service_sssd('stop')
        tools.remove_sss_cache('/var/lib/sss/db')
        configure_proxy(multihost)
        domain_params = {'allowed_uids': 'root, 5000',
                         'user_attributes': '-name, -uidNumber'}
        tools.sssd_conf("ifp", domain_params)
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl user-checks user5000'
        cmd = multihost.client[0].run_command(sssctl_cmd,
                                              raiseonerr=False)
        assert cmd.returncode == 0
        checks = ['SSSD InfoPipe user lookup', 'name: not set',
                  'uidNumber: not set']
        for _ in checks:
            find = re.compile(r'%s' % _)
            result = find.search(cmd.stdout_text)
            assert result is not None

    @pytest.mark.tier1_2
    def test_0008_bz1761047(self, multihost):
        """
        :title: sssctl: Null dereference in
         sssctl/sssctl_domains.c:sssctl_domain_status_active_server()
        :id: bf38d933-5eaf-43cc-b763-55cacf447bd1
        """
        multihost.client[0].run_command("yum install -y  nss-pam-ldapd")
        tools = sssdTools(multihost.client[0])
        domain_name = tools.get_domain_section_name()
        multihost.client[0].run_command("systemctl stop sssd")
        multihost.client[0].run_command("rm -rf /var/lib/sss/db/*")
        sssd_params = {'domains': '%s, %s' % (domain_name, 'proxy')}
        tools.sssd_conf("sssd", sssd_params, action='update')
        proxy_params = {'auth_provider': 'proxy',
                        'id_provider': 'proxy',
                        'debug_level': '0xFFF0',
                        'proxy_lib_name': 'ldap',
                        'proxy_pam_target': 'sssdproxyldap'}
        tools.sssd_conf("domain/proxy", proxy_params, action='add')
        multihost.client[0].service_sssd('start')
        cat = 'cat /etc/sssd/sssd.conf'
        multihost.client[0].run_command(cat, raiseonerr=False)
        sssctl_cmd = 'sssctl domain-status proxy'
        cmd = multihost.client[0].run_command(sssctl_cmd, raiseonerr=False)
        # when sssd crashes it returns with exit code of 139
        assert cmd.returncode == 0
        # remove the proxy section
        tools.sssd_conf("domain/proxy", proxy_params, 'delete')
        multihost.client[0].run_command(cat, raiseonerr=False)

    @pytest.mark.tier1_2
    def test_0009_bz1751691(self, multihost):
        """
        :title: IDM-SSSD-TC: sssctl: sssctl domain-list command displays
         results intermittently
        :id: 09fed728-1631-44fa-ad5d-082cba4a8ea2
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].run_command("systemctl stop sssd")
        multihost.client[0].run_command("rm -rf /var/lib/sss/db/*")
        tools.sssd_conf("sssd", {'services': 'nss, pam, ifp'},
                        action='update')
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl domain-list'
        checks = ['example1']
        for _ in range(10):
            time.sleep(5)
            cmd = multihost.client[0].run_command(sssctl_cmd,
                                                  raiseonerr=False)
            assert cmd.returncode == 0
            for _ in checks:
                find = re.compile(r'%s' % _)
                result = find.search(cmd.stdout_text)
                assert result is not None

    @pytest.mark.tier1_2
    def test_0010_bz1628122(self, multihost):
        """
        :title: sssctl: Printing incorrect information
         about domain with sssctl utility
        :id: 6997a8a4-0531-4e51-a10b-8c1d5791b67b
        """
        tools = sssdTools(multihost.client[0])
        multihost.client[0].run_command("systemctl stop sssd")
        multihost.client[0].run_command("rm -rf /var/lib/sss/db/*")
        tools.remove_sss_cache('/var/log/sssd')
        multihost.client[0].service_sssd('start')
        sssctl_cmd = 'sssctl domain-status %s -o' % ds_instance_name
        sssctl_cmd = multihost.client[0].run_command(sssctl_cmd,
                                                     raiseonerr=False)
        cmd = sssctl_cmd.stdout_text.split()[-1]
        log = 'cat /var/log/sssd/sssd_%s.log' % ds_instance_name
        log = multihost.client[0].run_command(log, raiseonerr=False)
        assert log.returncode == 0
        if 'Back end is online' or \
                'Backend is already online' in log.stdout_text:
            status = 'Online'
        else:
            status = 'Offline'
        if cmd == status:
            assert True
        else:
            if sssctl_cmd.returncode == 1:
                assert False, 'Invalid domain name'
            else:
                assert False, 'domain status conflict'

    @pytest.mark.tier1_2
    def test_0011_bz1406678(self, multihost):
        """
        :title: sssctl: sssd started before network
         sssd to go online once network service starts after it
        :id: f734660f-269e-49fd-9864-00de54b11b2c
        :customerscenario: True
        """
        multihost.client[0].run_command("systemctl stop sssd")
        multihost.client[0].run_command("rm -rf /var/lib/sss/db/*")
        multihost.client[0].service_sssd('start')
        stop_ds = 'systemctl stop dirsrv@%s' % ds_instance_name
        multihost.master[0].run_command(stop_ds, raiseonerr=False)
        status_ds = 'systemctl status dirsrv@%s' % ds_instance_name
        cmd = multihost.master[0].run_command(status_ds, raiseonerr=False)
        find = re.compile(r'slapd stopped')
        find.search(cmd.stdout_text)
        sss_kill = 'kill -10 `pidof sssd`'
        multihost.client[0].run_command(sss_kill, raiseonerr=False)
        domain_status = 'sssctl domain-status %s' % ds_instance_name
        cmd = multihost.client[0].run_command(domain_status,
                                              raiseonerr=False)
        find = re.compile(r'Online status: Offline')
        result = find.search(cmd.stdout_text)
        assert result is not None
        time.sleep(1)
        touch = 'touch /etc/resolv.conf'
        multihost.client[0].run_command(touch, raiseonerr=False)
        time.sleep(3)
        start_ds = 'systemctl start dirsrv@%s' % ds_instance_name
        multihost.master[0].run_command(start_ds, raiseonerr=False)
        cmd = multihost.master[0].run_command(status_ds, raiseonerr=False)
        find = re.compile(r'slapd started')
        find.search(cmd.stdout_text)
        time.sleep(6)
        domain_status = 'sssctl domain-status %s' % ds_instance_name
        cmd = multihost.client[0].run_command(domain_status,
                                              raiseonerr=False)
        find = re.compile(r'Online status: Online')
        result = find.search(cmd.stdout_text)
        assert result is not None
